! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! Linear Algebra Data and Routines File
! 
! Generated by KPP-2.2.1_rs5 symbolic chemistry Kinetics PreProcessor
!       (http://www.cs.vt.edu/~asandu/Software/KPP)
! KPP is distributed under GPL, the general public licence
!       (http://www.gnu.org/copyleft/gpl.html)
! (C) 1995-1997, V. Damian & A. Sandu, CGRER, Univ. Iowa
! (C) 1997-2005, A. Sandu, Michigan Tech, Virginia Tech
!     With important contributions from:
!        M. Damian, Villanova University, USA
!        R. Sander, Max-Planck Institute for Chemistry, Mainz, Germany
! 
! File                 : gckpp_LinearAlgebra.f90
! Time                 : Tue Jul  7 16:27:27 2015
! Working directory    : /home/mpayer/KPP
! Equation file        : gckpp.kpp
! Output root filename : gckpp
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



MODULE gckpp_LinearAlgebra

  USE gckpp_Parameters
  USE gckpp_JacobianSP

  IMPLICIT NONE

CONTAINS


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! SPARSE_UTIL - SPARSE utility functions
!   Arguments :
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppDecomp( JVS, IER )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Sparse LU factorization
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER  :: IER
      REAL(kind=dp) :: JVS(LU_NONZERO), W(NVAR), a
      INTEGER  :: k, kk, j, jj

      a = 0. ! mz_rs_20050606
      IER = 0
      DO k=1,NVAR
        ! mz_rs_20050606: don't check if real value == 0
        ! IF ( JVS( LU_DIAG(k) ) .EQ. 0. ) THEN
        IF ( ABS(JVS(LU_DIAG(k))) < TINY(a) ) THEN
            IER = k
            RETURN
        END IF
        DO kk = LU_CROW(k), LU_CROW(k+1)-1
              W( LU_ICOL(kk) ) = JVS(kk)
        END DO
        DO kk = LU_CROW(k), LU_DIAG(k)-1
            j = LU_ICOL(kk)
            a = -W(j) / JVS( LU_DIAG(j) )
            W(j) = -a
            DO jj = LU_DIAG(j)+1, LU_CROW(j+1)-1
               W( LU_ICOL(jj) ) = W( LU_ICOL(jj) ) + a*JVS(jj)
            END DO
         END DO
         DO kk = LU_CROW(k), LU_CROW(k+1)-1
            JVS(kk) = W( LU_ICOL(kk) )
         END DO
      END DO
      
END SUBROUTINE KppDecomp


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppDecompCmplx( JVS, IER )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Sparse LU factorization, complex
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER        :: IER
      DOUBLE COMPLEX :: JVS(LU_NONZERO), W(NVAR), a
      REAL(kind=dp)  :: b = 0.0
      INTEGER        :: k, kk, j, jj

      IER = 0
      DO k=1,NVAR
        IF ( ABS(JVS(LU_DIAG(k))) < TINY(b) ) THEN
            IER = k
            RETURN
        END IF
        DO kk = LU_CROW(k), LU_CROW(k+1)-1
              W( LU_ICOL(kk) ) = JVS(kk)
        END DO
        DO kk = LU_CROW(k), LU_DIAG(k)-1
            j = LU_ICOL(kk)
            a = -W(j) / JVS( LU_DIAG(j) )
            W(j) = -a
            DO jj = LU_DIAG(j)+1, LU_CROW(j+1)-1
               W( LU_ICOL(jj) ) = W( LU_ICOL(jj) ) + a*JVS(jj)
            END DO
         END DO
         DO kk = LU_CROW(k), LU_CROW(k+1)-1
            JVS(kk) = W( LU_ICOL(kk) )
         END DO
      END DO
      
END SUBROUTINE KppDecompCmplx


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppDecompCmplxR( JVSR, JVSI, IER )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!    Sparse LU factorization, complex
!   (Real and Imaginary parts are used instead of complex data type)     
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER       :: IER
      REAL(kind=dp) :: JVSR(LU_NONZERO), JVSI(LU_NONZERO) 
      REAL(kind=dp) :: WR(NVAR), WI(NVAR), ar, ai, den
      INTEGER       :: k, kk, j, jj

      IER = 0
      ar  = 0.0
      DO k=1,NVAR
        IF (  ( ABS(JVSR(LU_DIAG(k))) < TINY(ar) ) .AND. &
              ( ABS(JVSI(LU_DIAG(k))) < TINY(ar) ) )  THEN
            IER = k
            RETURN
        END IF
        DO kk = LU_CROW(k), LU_CROW(k+1)-1
              WR( LU_ICOL(kk) ) = JVSR(kk)
              WI( LU_ICOL(kk) ) = JVSI(kk)
        END DO
        DO kk = LU_CROW(k), LU_DIAG(k)-1
            j = LU_ICOL(kk)
            den = JVSR(LU_DIAG(j))**2 + JVSI(LU_DIAG(j))**2
            ar = -(WR(j)*JVSR(LU_DIAG(j)) + WI(j)*JVSI(LU_DIAG(j)))/den
            ai = -(WI(j)*JVSR(LU_DIAG(j)) - WR(j)*JVSI(LU_DIAG(j)))/den
            WR(j) = -ar
            WI(j) = -ai
            DO jj = LU_DIAG(j)+1, LU_CROW(j+1)-1
               WR( LU_ICOL(jj) ) = WR( LU_ICOL(jj) ) + ar*JVSR(jj) - ai*JVSI(jj)
               WI( LU_ICOL(jj) ) = WI( LU_ICOL(jj) ) + ar*JVSI(jj) + ai*JVSR(jj)
            END DO
         END DO
         DO kk = LU_CROW(k), LU_CROW(k+1)-1
            JVSR(kk) = WR( LU_ICOL(kk) )
            JVSI(kk) = WI( LU_ICOL(kk) )
         END DO
      END DO

END SUBROUTINE KppDecompCmplxR


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveIndirect( JVS, X )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Sparse solve subroutine using indirect addressing
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER  :: i, j
      REAL(kind=dp) :: JVS(LU_NONZERO), X(NVAR), sum

      DO i=1,NVAR
         DO j = LU_CROW(i), LU_DIAG(i)-1 
             X(i) = X(i) - JVS(j)*X(LU_ICOL(j));
         END DO  
      END DO

      DO i=NVAR,1,-1
        sum = X(i);
        DO j = LU_DIAG(i)+1, LU_CROW(i+1)-1
          sum = sum - JVS(j)*X(LU_ICOL(j));
        END DO
        X(i) = sum/JVS(LU_DIAG(i));
      END DO
      
END SUBROUTINE KppSolveIndirect


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveTRIndirect( JVS, X )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Complex sparse solve transpose subroutine using indirect addressing
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER       :: i, j
      REAL(kind=dp) :: JVS(LU_NONZERO), X(NVAR)

      DO i=1,NVAR
        X(i) = X(i)/JVS(LU_DIAG(i))
        ! subtract all nonzero elements in row i of JVS from X
        DO j=LU_DIAG(i)+1,LU_CROW(i+1)-1
          X(LU_ICOL(j)) = X(LU_ICOL(j))-JVS(j)*X(i)
        END DO
      END DO

      DO i=NVAR, 1, -1
        ! subtract all nonzero elements in row i of JVS from X
        DO j=LU_CROW(i),LU_DIAG(i)-1
          X(LU_ICOL(j)) = X(LU_ICOL(j))-JVS(j)*X(i)
        END DO
      END DO
      
END SUBROUTINE KppSolveTRIndirect


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveCmplx( JVS, X )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Complex sparse solve subroutine using indirect addressing
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER        :: i, j
      DOUBLE COMPLEX :: JVS(LU_NONZERO), X(NVAR), sum

      DO i=1,NVAR
         DO j = LU_CROW(i), LU_DIAG(i)-1 
             X(i) = X(i) - JVS(j)*X(LU_ICOL(j));
         END DO  
      END DO

      DO i=NVAR,1,-1
        sum = X(i);
        DO j = LU_DIAG(i)+1, LU_CROW(i+1)-1
          sum = sum - JVS(j)*X(LU_ICOL(j));
        END DO
        X(i) = sum/JVS(LU_DIAG(i));
      END DO
      
END SUBROUTINE KppSolveCmplx

! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveCmplxR( JVSR, JVSI, XR, XI )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!   Complex sparse solve subroutine using indirect addressing
!   (Real and Imaginary parts are used instead of complex data type)     
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER       ::  i, j
      REAL(kind=dp) ::  JVSR(LU_NONZERO), JVSI(LU_NONZERO), XR(NVAR), XI(NVAR), sumr, sumi, den

      DO i=1,NVAR
         DO j = LU_CROW(i), LU_DIAG(i)-1 
             XR(i) = XR(i) - (JVSR(j)*XR(LU_ICOL(j)) - JVSI(j)*XI(LU_ICOL(j)))
             XI(i) = XI(i) - (JVSR(j)*XI(LU_ICOL(j)) + JVSI(j)*XR(LU_ICOL(j)))
         END DO  
      END DO

      DO i=NVAR,1,-1
        sumr = XR(i); sumi = XI(i)
        DO j = LU_DIAG(i)+1, LU_CROW(i+1)-1
            sumr = sumr - (JVSR(j)*XR(LU_ICOL(j)) - JVSI(j)*XI(LU_ICOL(j)))
            sumi = sumi - (JVSR(j)*XI(LU_ICOL(j)) + JVSI(j)*XR(LU_ICOL(j)))
        END DO
        den   = JVSR(LU_DIAG(i))**2 + JVSI(LU_DIAG(i))**2
        XR(i) = (sumr*JVSR(LU_DIAG(i)) + sumi*JVSI(LU_DIAG(i)))/den
        XI(i) = (sumi*JVSR(LU_DIAG(i)) - sumr*JVSI(LU_DIAG(i)))/den
      END DO
      
END SUBROUTINE KppSolveCmplxR


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveTRCmplx( JVS, X )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Complex sparse solve transpose subroutine using indirect addressing
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER        :: i, j
      DOUBLE COMPLEX :: JVS(LU_NONZERO), X(NVAR)

      DO i=1,NVAR
        X(i) = X(i)/JVS(LU_DIAG(i))
        ! subtract all nonzero elements in row i of JVS from X
        DO j=LU_DIAG(i)+1,LU_CROW(i+1)-1
          X(LU_ICOL(j)) = X(LU_ICOL(j))-JVS(j)*X(i)
        END DO
      END DO

      DO i=NVAR, 1, -1
        ! subtract all nonzero elements in row i of JVS from X
        DO j=LU_CROW(i),LU_DIAG(i)-1
          X(LU_ICOL(j)) = X(LU_ICOL(j))-JVS(j)*X(i)
        END DO
      END DO
      
END SUBROUTINE KppSolveTRCmplx


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveTRCmplxR( JVSR, JVSI, XR, XI )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!   Complex sparse solve transpose subroutine using indirect addressing
!   (Real and Imaginary parts are used instead of complex data type)     
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER       ::  i, j
      REAL(kind=dp) ::  JVSR(LU_NONZERO), JVSI(LU_NONZERO), XR(NVAR), XI(NVAR), den

      DO i=1,NVAR
        den   = JVSR(LU_DIAG(i))**2 + JVSI(LU_DIAG(i))**2
        XR(i) = (XR(i)*JVSR(LU_DIAG(i)) + XI(i)*JVSI(LU_DIAG(i)))/den
        XI(i) = (XI(i)*JVSR(LU_DIAG(i)) - XR(i)*JVSI(LU_DIAG(i)))/den
        ! subtract all nonzero elements in row i of JVS from X
        DO j=LU_DIAG(i)+1,LU_CROW(i+1)-1
          XR(LU_ICOL(j)) = XR(LU_ICOL(j))-(JVSR(j)*XR(i) - JVSI(j)*XI(i))
          XI(LU_ICOL(j)) = XI(LU_ICOL(j))-(JVSI(j)*XR(i) + JVSR(j)*XI(i))
        END DO
      END DO

      DO i=NVAR, 1, -1
        ! subtract all nonzero elements in row i of JVS from X
        DO j=LU_CROW(i),LU_DIAG(i)-1
          XR(LU_ICOL(j)) = XR(LU_ICOL(j))-(JVSR(j)*XR(i) - JVSI(j)*XI(i))
          XI(LU_ICOL(j)) = XI(LU_ICOL(j))-(JVSI(j)*XR(i) + JVSR(j)*XI(i))
        END DO
      END DO
      
END SUBROUTINE KppSolveTRCmplxR


!
! Next few commented subroutines perform sparse big linear algebra
!
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!SUBROUTINE KppDecompBig( JVS, IP, IER )
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!        Sparse LU factorization
!!        for the Runge Kutta (3n)x(3n) linear system
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
!  USE gckpp_Parameters
!  USE gckpp_JacobianSP
!
!      INTEGER  :: IP3(3), IER, IP(3,NVAR)
!      REAL(kind=dp) :: JVS(3,3,LU_NONZERO), W(3,3,NVAR), a(3,3), E(3,3)
!      INTEGER  :: k, kk, j, jj
!
!      a = 0.0d0
!      IER = 0
!      DO k=1,NVAR
!        DO kk = LU_CROW(k), LU_CROW(k+1)-1
!              W( 1:3,1:3,LU_ICOL(kk) ) = JVS(1:3,1:3,kk)
!        END DO
!        DO kk = LU_CROW(k), LU_DIAG(k)-1
!            j = LU_ICOL(kk)
!            E(1:3,1:3) = JVS( 1:3,1:3,LU_DIAG(j) )
!            ! CALL DGETRF(3,3,E,3,IP3,IER) 
!            CALL FAC3(E,IP3,IER)
!            IF ( IER /= 0 )  RETURN
!            ! a = W(j) / JVS( LU_DIAG(j) )
!            a(1:3,1:3) = W( 1:3,1:3,j )
!            ! CALL DGETRS ('N',3,3,E,3,IP3,a,3,IER) 
!            CALL SOL3('N',E,IP3,a(1,1))
!            CALL SOL3('N',E,IP3,a(1,2))
!            CALL SOL3('N',E,IP3,a(1,3))
!            W(1:3,1:3,j) = a(1:3,1:3)
!            DO jj = LU_DIAG(j)+1, LU_CROW(j+1)-1
!               W( 1:3,1:3,LU_ICOL(jj) ) = W( 1:3,1:3,LU_ICOL(jj) ) &
!                        - MATMUL( a(1:3,1:3) , JVS(1:3,1:3,jj) )
!            END DO
!         END DO
!         DO kk = LU_CROW(k), LU_CROW(k+1)-1
!            JVS(1:3,1:3,kk) = W( 1:3,1:3,LU_ICOL(kk) )
!         END DO
!      END DO
!
!      DO k=1,NVAR
!         ! CALL WGEFA(JVS(1,1,LU_DIAG(k)),3,3,IP(1,k),IER)
!         ! CALL DGETRF(3,3,JVS(1,1,LU_DIAG(k)),3,IP(1,k),IER)
!         CALL FAC3(JVS(1,1,LU_DIAG(k)),IP(1,k),IER)
!         IF ( IER /= 0 )  RETURN
!      END DO 
!      
!END SUBROUTINE KppDecompBig
!
!
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!SUBROUTINE KppSolveBig( JVS, IP, X )
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!        Sparse solve subroutine using indirect addressing
!!        for the Runge Kutta (3n)x(3n) linear system
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
!  USE gckpp_Parameters
!  USE gckpp_JacobianSP
!
!      INTEGER  :: i, j, k, m, IP3(3), IP(3,NVAR), IER
!      REAL(kind=dp) :: JVS(3,3,LU_NONZERO), X(3,NVAR), sum(3)
!
!      DO i=1,NVAR
!        DO j = LU_CROW(i), LU_DIAG(i)-1 
!          !X(1:3,i) = X(1:3,i) - MATMUL(JVS(1:3,1:3,j),X(1:3,LU_ICOL(j)));
!          DO k=1,3
!            DO m=1,3
!              X(k,i) = X(k,i) - JVS(k,m,j)*X(m,LU_ICOL(j))
!            END DO
!          END DO
!        END DO  
!      END DO
!
!      DO i=NVAR,1,-1
!        sum(1:3) = X(1:3,i);
!        DO j = LU_DIAG(i)+1, LU_CROW(i+1)-1
!          !sum(1:3) = sum(1:3) - MATMUL(JVS(1:3,1:3,j),X(1:3,LU_ICOL(j)));
!          DO k=1,3
!            DO m=1,3
!              sum(k) = sum(k) - JVS(k,m,j)*X(m,LU_ICOL(j))
!            END DO
!          END DO
!        END DO
!        ! X(i) = sum/JVS(LU_DIAG(i));
!        ! CALL DGETRS ('N',3,1,JVS(1:3,1:3,LU_DIAG(i)),3,IP(1,i),sum,3,0) 
!        ! CALL WGESL('N',JVS(1,1,LU_DIAG(i)),3,3,IP(1,i),sum)
!        CALL SOL3('N',JVS(1,1,LU_DIAG(i)),IP(1,i),sum)
!        X(1:3,i) = sum(1:3)
!      END DO
!      
!END SUBROUTINE KppSolveBig
!
!
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!SUBROUTINE KppSolveBigTR( JVS, IP, X )
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!        Big sparse transpose solve using indirect addressing
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
!  USE gckpp_Parameters
!  USE gckpp_JacobianSP
!
!      INTEGER       :: i, j, k, m, IP(3,NVAR)
!      REAL(kind=dp) :: JVS(3,3,LU_NONZERO), X(3,NVAR)
!
!      DO i=1,NVAR
!        ! X(i) = X(i)/JVS(LU_DIAG(i))
!        CALL SOL3('T',JVS(1,1,LU_DIAG(i)),IP(1,i),X(1,i))
!        DO j=LU_DIAG(i)+1,LU_CROW(i+1)-1
!         !X(1:3,LU_ICOL(j)) = X(1:3,LU_ICOL(j)) &
!          !    - MATMUL( TRANSPOSE(JVS(1:3,1:3,j)), X(1:3,i) )
!          DO k=1,3
!            DO m=1,3
!              X(k,LU_ICOL(j)) = X(k,LU_ICOL(j)) - JVS(m,k,j)*X(m,i)
!            END DO
!          END DO
!       END DO
!      END DO
!
!      DO i=NVAR, 1, -1
!        DO j=LU_CROW(i),LU_DIAG(i)-1
!         !X(1:3,LU_ICOL(j)) = X(1:3,LU_ICOL(j)) &
!          !   - MATMUL( TRANSPOSE(JVS(1:3,1:3,j)), X(1:3,i) )
!          DO k=1,3
!            DO m=1,3
!              X(k,LU_ICOL(j)) = X(k,LU_ICOL(j)) - JVS(m,k,j)*X(m,i)
!            END DO
!          END DO
!       END DO
!      END DO
!      
!END SUBROUTINE KppSolveBigTR
!
!
!
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!SUBROUTINE FAC3(A,IPVT,INFO)
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!     FAC3 FACTORS THE MATRIX A (3,3) BY
!!           GAUSS ELIMINATION WITH PARTIAL PIVOTING
!!     LINPACK - LIKE 
!!
!!     Remove comments to perform pivoting
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!
!      REAL(kind=dp) :: A(3,3)
!      INTEGER       :: IPVT(3),INFO
!!      INTEGER       :: L
!!      REAL(kind=dp) :: t, dmax, da, TMP(3)
!      REAL(kind=dp), PARAMETER :: ZERO = 0.0, ONE = 1.0
!
!      info = 0
!!      t = TINY(da)
!!      
!!      da = ABS(A(1,1)); L = 1
!!      IF ( ABS(A(2,1))>da ) THEN
!!        da = ABS(A(2,1)); L = 2
!!        IF ( ABS(A(3,1))>da ) THEN
!!          L = 3
!!        END IF  
!!      END IF  
!!      IPVT(1)  = L
!!      IF (L /=1 ) THEN
!!         TMP(1:3) = A(L,1:3)
!!         A(L,1:3) = A(1,1:3)
!!         A(1,1:3) = TMP(1:3)
!!      END IF
!!      IF (ABS(A(1,1)) < t) THEN
!!         info = 1
!!         return
!!      END IF   
!!
!      A(2,1) = A(2,1)/A(1,1)
!      A(2,2) = A(2,2) - A(2,1)*A(1,2)
!      A(2,3) = A(2,3) - A(2,1)*A(1,3)
!      A(3,1) = A(3,1)/A(1,1)
!      A(3,2) = A(3,2) - A(3,1)*A(1,2)
!      A(3,3) = A(3,3) - A(3,1)*A(1,3)
!      
!!      IPVT(2)  = 2
!!      IF (ABS(A(3,2))>ABS(A(2,2))) THEN
!!         IPVT(2)  = 3
!!         TMP(2:3) = A(3,2:3)
!!         A(3,2:3) = A(2,2:3)
!!         A(2,2:3) = TMP(2:3)
!!      END IF
!!      IF (ABS(A(2,2)) < t) THEN
!!         info = 1
!!         return
!!      END IF   
!!      
!      A(3,2)   = A(3,2)/A(2,2)
!      A(3,3)   = A(3,3) - A(3,2)*A(2,3)
!      IPVT(3)  = 3
!      
!END SUBROUTINE FAC3
!
!
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!SUBROUTINE SOL3(Trans,A,IPVT,b)
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!     SOL3 solves the system 3x3
!!     A * x = b  or  trans(a) * x = b
!!     using the factors computed by WGEFA.
!!
!!     Trans      = 'N'   to solve  A*x = b ,
!!                = 'T'   to solve  transpose(A)*x = b
!!     LINPACK - LIKE 
!!
!!     Remove comments to use pivoting
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
!      CHARACTER     :: Trans
!      REAL(kind=dp) :: a(3,3),b(3)
!      INTEGER       :: IPVT(3)
!!      INTEGER       :: L
!!      REAL(kind=dp) :: TMP
!      
!      SELECT CASE (Trans)
!
!      CASE ('n','N')  !  Solve  A * x = b
!
!!     Solve  L*y = b
!!         L = IPVT(1)
!!         IF (L /= 1) THEN
!!            TMP = B(1); B(1) = B(L); B(L) = TMP
!!         END IF
!         b(2) = b(2)-A(2,1)*b(1)
!         b(3) = b(3)-A(3,1)*b(1)
!         
!!         L = IPVT(2)
!!         IF (L /= 2) THEN
!!            TMP = B(2); B(2) = B(L); B(L) = TMP
!!         END IF
!         b(3) = b(3)-A(3,2)*b(2)
!
!!     Solve  U*x = y
!         b(3) = b(3)/A(3,3)
!         b(2) = (b(2)-A(2,3)*b(3))/A(2,2)
!         b(1) = (b(1)-A(1,3)*b(3)-A(1,2)*b(2))/A(1,1)
!      
!      
!      CASE ('t','T')  !  Solve transpose(A) * x = b
!
!!      Solve transpose(U)*y = b
!         b(1) = b(1)/A(1,1)
!         b(2) = (b(2)-A(1,2)*b(1))/A(2,2)
!         b(3) = (b(3)-A(1,3)*b(1)-A(2,3)*b(2))/A(3,3)
!
!!      Solve transpose(L)*x = y
!         b(2) = b(2)-A(3,2)*b(3)
!!         L = ipvt(2)
!!         IF (L /= 2) THEN
!!            TMP = B(2); B(2) = B(L); B(L) = TMP
!!         END IF
!         b(1) = b(1)-A(3,1)*b(3)-A(2,1)*b(2)
!!         L = ipvt(1)
!!         IF (L /= 1) THEN
!!            TMP = B(1); B(1) = B(L); B(L) = TMP
!!         END IF
!   
!      END SELECT
!
!END SUBROUTINE SOL3

! End of SPARSE_UTIL function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! KppSolve - sparse back substitution
!   Arguments :
!      JVS       - sparse Jacobian of variables
!      X         - Vector for variables
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SUBROUTINE KppSolve ( JVS, X )

! JVS - sparse Jacobian of variables
  REAL(kind=dp) :: JVS(LU_NONZERO)
! X - Vector for variables
  REAL(kind=dp) :: X(NVAR)

  X(59) = X(59)-JVS(180)*X(53)
  X(60) = X(60)-JVS(185)*X(48)
  X(61) = X(61)-JVS(190)*X(49)
  X(77) = X(77)-JVS(252)*X(76)
  X(78) = X(78)-JVS(260)*X(70)
  X(92) = X(92)-JVS(328)*X(91)
  X(98) = X(98)-JVS(362)*X(97)
  X(99) = X(99)-JVS(368)*X(56)-JVS(369)*X(90)
  X(100) = X(100)-JVS(375)*X(57)
  X(101) = X(101)-JVS(382)*X(93)
  X(105) = X(105)-JVS(416)*X(96)
  X(106) = X(106)-JVS(425)*X(74)
  X(108) = X(108)-JVS(442)*X(82)
  X(109) = X(109)-JVS(451)*X(70)-JVS(452)*X(85)-JVS(453)*X(86)-JVS(454)*X(91)-JVS(455)*X(92)-JVS(456)*X(97)-JVS(457)&
             &*X(98)-JVS(458)*X(99)-JVS(459)*X(104)-JVS(460)*X(106)
  X(110) = X(110)-JVS(492)*X(89)-JVS(493)*X(91)-JVS(494)*X(99)-JVS(495)*X(103)-JVS(496)*X(105)
  X(113) = X(113)-JVS(524)*X(62)-JVS(525)*X(64)-JVS(526)*X(69)-JVS(527)*X(82)-JVS(528)*X(85)-JVS(529)*X(101)-JVS(530)&
             &*X(103)-JVS(531)*X(108)
  X(114) = X(114)-JVS(554)*X(96)
  X(115) = X(115)-JVS(564)*X(70)-JVS(565)*X(72)
  X(116) = X(116)-JVS(575)*X(79)
  X(117) = X(117)-JVS(583)*X(81)-JVS(584)*X(112)
  X(118) = X(118)-JVS(597)*X(90)-JVS(598)*X(96)-JVS(599)*X(112)
  X(119) = X(119)-JVS(610)*X(99)-JVS(611)*X(103)-JVS(612)*X(107)-JVS(613)*X(110)-JVS(614)*X(111)-JVS(615)*X(112)&
             &-JVS(616)*X(118)
  X(120) = X(120)-JVS(632)*X(89)
  X(121) = X(121)-JVS(640)*X(86)
  X(124) = X(124)-JVS(662)*X(81)-JVS(663)*X(98)
  X(125) = X(125)-JVS(673)*X(70)-JVS(674)*X(74)
  X(126) = X(126)-JVS(684)*X(84)-JVS(685)*X(98)
  X(127) = X(127)-JVS(695)*X(87)-JVS(696)*X(89)-JVS(697)*X(91)-JVS(698)*X(99)-JVS(699)*X(100)-JVS(700)*X(103)-JVS(701)&
             &*X(105)-JVS(702)*X(114)-JVS(703)*X(117)-JVS(704)*X(118)-JVS(705)*X(120)-JVS(706)*X(121)-JVS(707)*X(123)&
             &-JVS(708)*X(124)
  X(129) = X(129)-JVS(736)*X(71)-JVS(737)*X(85)
  X(130) = X(130)-JVS(751)*X(54)-JVS(752)*X(80)-JVS(753)*X(122)
  X(131) = X(131)-JVS(766)*X(95)-JVS(767)*X(96)-JVS(768)*X(123)
  X(132) = X(132)-JVS(780)*X(87)-JVS(781)*X(106)-JVS(782)*X(125)-JVS(783)*X(128)
  X(134) = X(134)-JVS(803)*X(1)-JVS(804)*X(68)-JVS(805)*X(69)-JVS(806)*X(75)-JVS(807)*X(79)-JVS(808)*X(83)-JVS(809)&
             &*X(84)-JVS(810)*X(86)-JVS(811)*X(87)-JVS(812)*X(89)-JVS(813)*X(90)-JVS(814)*X(96)-JVS(815)*X(97)-JVS(816)&
             &*X(98)-JVS(817)*X(99)-JVS(818)*X(100)-JVS(819)*X(104)-JVS(820)*X(105)-JVS(821)*X(107)-JVS(822)*X(110)-JVS(823)&
             &*X(111)-JVS(824)*X(112)-JVS(825)*X(114)-JVS(826)*X(115)-JVS(827)*X(116)-JVS(828)*X(117)-JVS(829)*X(118)&
             &-JVS(830)*X(119)-JVS(831)*X(120)-JVS(832)*X(121)-JVS(833)*X(122)-JVS(834)*X(123)-JVS(835)*X(124)-JVS(836)&
             &*X(125)-JVS(837)*X(126)-JVS(838)*X(128)-JVS(839)*X(129)-JVS(840)*X(130)-JVS(841)*X(131)-JVS(842)*X(132)&
             &-JVS(843)*X(133)
  X(135) = X(135)-JVS(868)*X(71)-JVS(869)*X(80)-JVS(870)*X(84)-JVS(871)*X(98)-JVS(872)*X(122)-JVS(873)*X(124)-JVS(874)&
             &*X(126)-JVS(875)*X(128)-JVS(876)*X(129)-JVS(877)*X(130)-JVS(878)*X(133)
  X(136) = X(136)-JVS(896)*X(90)-JVS(897)*X(97)
  X(137) = X(137)-JVS(908)*X(91)-JVS(909)*X(118)
  X(138) = X(138)-JVS(920)*X(90)-JVS(921)*X(97)
  X(139) = X(139)-JVS(930)*X(95)-JVS(931)*X(112)-JVS(932)*X(123)-JVS(933)*X(131)-JVS(934)*X(138)
  X(140) = X(140)-JVS(945)*X(84)-JVS(946)*X(86)-JVS(947)*X(91)-JVS(948)*X(92)-JVS(949)*X(99)-JVS(950)*X(100)-JVS(951)&
             &*X(103)-JVS(952)*X(104)-JVS(953)*X(112)-JVS(954)*X(114)-JVS(955)*X(116)-JVS(956)*X(118)-JVS(957)*X(121)&
             &-JVS(958)*X(122)-JVS(959)*X(126)-JVS(960)*X(131)-JVS(961)*X(132)-JVS(962)*X(133)-JVS(963)*X(136)-JVS(964)&
             &*X(137)-JVS(965)*X(138)-JVS(966)*X(139)
  X(141) = X(141)-JVS(983)*X(64)-JVS(984)*X(88)-JVS(985)*X(101)-JVS(986)*X(103)-JVS(987)*X(137)-JVS(988)*X(138)
  X(142) = X(142)-JVS(999)*X(94)-JVS(1000)*X(97)-JVS(1001)*X(107)-JVS(1002)*X(111)-JVS(1003)*X(133)
  X(143) = X(143)-JVS(1012)*X(90)-JVS(1013)*X(97)-JVS(1014)*X(138)-JVS(1015)*X(142)
  X(144) = X(144)-JVS(1024)*X(79)-JVS(1025)*X(104)-JVS(1026)*X(122)-JVS(1027)*X(133)-JVS(1028)*X(142)-JVS(1029)*X(143)
  X(145) = X(145)-JVS(1039)*X(95)-JVS(1040)*X(123)-JVS(1041)*X(131)-JVS(1042)*X(138)-JVS(1043)*X(139)-JVS(1044)*X(142)&
             &-JVS(1045)*X(143)
  X(146) = X(146)-JVS(1055)*X(72)-JVS(1056)*X(81)-JVS(1057)*X(88)-JVS(1058)*X(94)-JVS(1059)*X(95)-JVS(1060)*X(101)&
             &-JVS(1061)*X(102)-JVS(1062)*X(103)-JVS(1063)*X(107)-JVS(1064)*X(111)-JVS(1065)*X(115)-JVS(1066)*X(116)&
             &-JVS(1067)*X(120)-JVS(1068)*X(121)-JVS(1069)*X(123)-JVS(1070)*X(124)-JVS(1071)*X(125)-JVS(1072)*X(126)&
             &-JVS(1073)*X(128)-JVS(1074)*X(131)-JVS(1075)*X(133)-JVS(1076)*X(136)-JVS(1077)*X(137)-JVS(1078)*X(138)&
             &-JVS(1079)*X(139)-JVS(1080)*X(141)-JVS(1081)*X(142)-JVS(1082)*X(143)-JVS(1083)*X(144)-JVS(1084)*X(145)
  X(147) = X(147)-JVS(1097)*X(93)-JVS(1098)*X(96)-JVS(1099)*X(97)-JVS(1100)*X(98)-JVS(1101)*X(104)-JVS(1102)*X(108)&
             &-JVS(1103)*X(112)-JVS(1104)*X(118)-JVS(1105)*X(122)-JVS(1106)*X(130)-JVS(1107)*X(133)-JVS(1108)*X(136)&
             &-JVS(1109)*X(137)-JVS(1110)*X(138)-JVS(1111)*X(142)-JVS(1112)*X(143)-JVS(1113)*X(144)-JVS(1114)*X(146)
  X(148) = X(148)-JVS(1126)*X(63)-JVS(1127)*X(67)-JVS(1128)*X(79)-JVS(1129)*X(87)-JVS(1130)*X(89)-JVS(1131)*X(97)&
             &-JVS(1132)*X(104)-JVS(1133)*X(105)-JVS(1134)*X(106)-JVS(1135)*X(115)-JVS(1136)*X(116)-JVS(1137)*X(120)&
             &-JVS(1138)*X(121)-JVS(1139)*X(122)-JVS(1140)*X(123)-JVS(1141)*X(124)-JVS(1142)*X(125)-JVS(1143)*X(126)&
             &-JVS(1144)*X(127)-JVS(1145)*X(128)-JVS(1146)*X(129)-JVS(1147)*X(130)-JVS(1148)*X(131)-JVS(1149)*X(132)&
             &-JVS(1150)*X(133)-JVS(1151)*X(135)-JVS(1152)*X(136)-JVS(1153)*X(137)-JVS(1154)*X(138)-JVS(1155)*X(139)&
             &-JVS(1156)*X(140)-JVS(1157)*X(141)-JVS(1158)*X(142)-JVS(1159)*X(143)-JVS(1160)*X(144)-JVS(1161)*X(145)&
             &-JVS(1162)*X(146)-JVS(1163)*X(147)
  X(149) = X(149)-JVS(1174)*X(89)-JVS(1175)*X(92)-JVS(1176)*X(93)-JVS(1177)*X(94)-JVS(1178)*X(107)-JVS(1179)*X(111)&
             &-JVS(1180)*X(116)-JVS(1181)*X(120)-JVS(1182)*X(121)-JVS(1183)*X(128)-JVS(1184)*X(132)-JVS(1185)*X(133)&
             &-JVS(1186)*X(136)-JVS(1187)*X(137)-JVS(1188)*X(138)-JVS(1189)*X(139)-JVS(1190)*X(141)-JVS(1191)*X(142)&
             &-JVS(1192)*X(143)-JVS(1193)*X(144)-JVS(1194)*X(145)-JVS(1195)*X(147)-JVS(1196)*X(148)
  X(150) = X(150)-JVS(1206)*X(48)-JVS(1207)*X(49)-JVS(1208)*X(50)-JVS(1209)*X(51)-JVS(1210)*X(52)-JVS(1211)*X(53)&
             &-JVS(1212)*X(56)-JVS(1213)*X(58)-JVS(1214)*X(64)-JVS(1215)*X(65)-JVS(1216)*X(66)-JVS(1217)*X(67)-JVS(1218)&
             &*X(69)-JVS(1219)*X(70)-JVS(1220)*X(71)-JVS(1221)*X(72)-JVS(1222)*X(73)-JVS(1223)*X(74)-JVS(1224)*X(75)&
             &-JVS(1225)*X(76)-JVS(1226)*X(77)-JVS(1227)*X(78)-JVS(1228)*X(79)-JVS(1229)*X(80)-JVS(1230)*X(81)-JVS(1231)&
             &*X(82)-JVS(1232)*X(83)-JVS(1233)*X(84)-JVS(1234)*X(85)-JVS(1235)*X(86)-JVS(1236)*X(87)-JVS(1237)*X(88)&
             &-JVS(1238)*X(89)-JVS(1239)*X(90)-JVS(1240)*X(91)-JVS(1241)*X(92)-JVS(1242)*X(93)-JVS(1243)*X(94)-JVS(1244)&
             &*X(95)-JVS(1245)*X(96)-JVS(1246)*X(97)-JVS(1247)*X(98)-JVS(1248)*X(99)-JVS(1249)*X(100)-JVS(1250)*X(101)&
             &-JVS(1251)*X(102)-JVS(1252)*X(103)-JVS(1253)*X(104)-JVS(1254)*X(105)-JVS(1255)*X(106)-JVS(1256)*X(107)&
             &-JVS(1257)*X(108)-JVS(1258)*X(109)-JVS(1259)*X(110)-JVS(1260)*X(111)-JVS(1261)*X(112)-JVS(1262)*X(113)&
             &-JVS(1263)*X(114)-JVS(1264)*X(115)-JVS(1265)*X(116)-JVS(1266)*X(117)-JVS(1267)*X(118)-JVS(1268)*X(119)&
             &-JVS(1269)*X(120)-JVS(1270)*X(121)-JVS(1271)*X(122)-JVS(1272)*X(123)-JVS(1273)*X(124)-JVS(1274)*X(125)&
             &-JVS(1275)*X(126)-JVS(1276)*X(127)-JVS(1277)*X(128)-JVS(1278)*X(129)-JVS(1279)*X(130)-JVS(1280)*X(131)&
             &-JVS(1281)*X(132)-JVS(1282)*X(133)-JVS(1283)*X(134)-JVS(1284)*X(135)-JVS(1285)*X(136)-JVS(1286)*X(137)&
             &-JVS(1287)*X(138)-JVS(1288)*X(139)-JVS(1289)*X(140)-JVS(1290)*X(141)-JVS(1291)*X(142)-JVS(1292)*X(143)&
             &-JVS(1293)*X(144)-JVS(1294)*X(145)-JVS(1295)*X(146)-JVS(1296)*X(147)-JVS(1297)*X(148)-JVS(1298)*X(149)
  X(151) = X(151)-JVS(1307)*X(63)-JVS(1308)*X(67)-JVS(1309)*X(68)-JVS(1310)*X(69)-JVS(1311)*X(75)-JVS(1312)*X(97)&
             &-JVS(1313)*X(106)-JVS(1314)*X(107)-JVS(1315)*X(111)-JVS(1316)*X(115)-JVS(1317)*X(116)-JVS(1318)*X(117)&
             &-JVS(1319)*X(120)-JVS(1320)*X(121)-JVS(1321)*X(123)-JVS(1322)*X(124)-JVS(1323)*X(125)-JVS(1324)*X(126)&
             &-JVS(1325)*X(128)-JVS(1326)*X(129)-JVS(1327)*X(130)-JVS(1328)*X(132)-JVS(1329)*X(133)-JVS(1330)*X(135)&
             &-JVS(1331)*X(136)-JVS(1332)*X(137)-JVS(1333)*X(138)-JVS(1334)*X(139)-JVS(1335)*X(140)-JVS(1336)*X(141)&
             &-JVS(1337)*X(142)-JVS(1338)*X(143)-JVS(1339)*X(144)-JVS(1340)*X(145)-JVS(1341)*X(146)-JVS(1342)*X(147)&
             &-JVS(1343)*X(148)-JVS(1344)*X(149)-JVS(1345)*X(150)
  X(152) = X(152)-JVS(1353)*X(58)-JVS(1354)*X(59)-JVS(1355)*X(60)-JVS(1356)*X(61)-JVS(1357)*X(65)-JVS(1358)*X(68)&
             &-JVS(1359)*X(69)-JVS(1360)*X(71)-JVS(1361)*X(72)-JVS(1362)*X(73)-JVS(1363)*X(74)-JVS(1364)*X(75)-JVS(1365)&
             &*X(80)-JVS(1366)*X(81)-JVS(1367)*X(84)-JVS(1368)*X(86)-JVS(1369)*X(88)-JVS(1370)*X(89)-JVS(1371)*X(90)&
             &-JVS(1372)*X(91)-JVS(1373)*X(92)-JVS(1374)*X(93)-JVS(1375)*X(94)-JVS(1376)*X(95)-JVS(1377)*X(96)-JVS(1378)&
             &*X(97)-JVS(1379)*X(98)-JVS(1380)*X(99)-JVS(1381)*X(100)-JVS(1382)*X(101)-JVS(1383)*X(102)-JVS(1384)*X(103)&
             &-JVS(1385)*X(107)-JVS(1386)*X(108)-JVS(1387)*X(109)-JVS(1388)*X(110)-JVS(1389)*X(111)-JVS(1390)*X(112)&
             &-JVS(1391)*X(114)-JVS(1392)*X(115)-JVS(1393)*X(116)-JVS(1394)*X(117)-JVS(1395)*X(118)-JVS(1396)*X(119)&
             &-JVS(1397)*X(120)-JVS(1398)*X(121)-JVS(1399)*X(122)-JVS(1400)*X(123)-JVS(1401)*X(124)-JVS(1402)*X(125)&
             &-JVS(1403)*X(126)-JVS(1404)*X(127)-JVS(1405)*X(128)-JVS(1406)*X(129)-JVS(1407)*X(130)-JVS(1408)*X(131)&
             &-JVS(1409)*X(132)-JVS(1410)*X(133)-JVS(1411)*X(134)-JVS(1412)*X(135)-JVS(1413)*X(136)-JVS(1414)*X(137)&
             &-JVS(1415)*X(138)-JVS(1416)*X(139)-JVS(1417)*X(140)-JVS(1418)*X(141)-JVS(1419)*X(142)-JVS(1420)*X(143)&
             &-JVS(1421)*X(144)-JVS(1422)*X(145)-JVS(1423)*X(146)-JVS(1424)*X(147)-JVS(1425)*X(148)-JVS(1426)*X(149)&
             &-JVS(1427)*X(150)-JVS(1428)*X(151)
  X(153) = X(153)-JVS(1435)*X(50)-JVS(1436)*X(51)-JVS(1437)*X(52)-JVS(1438)*X(55)-JVS(1439)*X(70)-JVS(1440)*X(76)&
             &-JVS(1441)*X(77)-JVS(1442)*X(78)-JVS(1443)*X(82)-JVS(1444)*X(85)-JVS(1445)*X(106)-JVS(1446)*X(108)-JVS(1447)&
             &*X(125)-JVS(1448)*X(134)-JVS(1449)*X(135)-JVS(1450)*X(136)-JVS(1451)*X(137)-JVS(1452)*X(138)-JVS(1453)*X(139)&
             &-JVS(1454)*X(140)-JVS(1455)*X(141)-JVS(1456)*X(142)-JVS(1457)*X(143)-JVS(1458)*X(144)-JVS(1459)*X(145)&
             &-JVS(1460)*X(146)-JVS(1461)*X(147)-JVS(1462)*X(148)-JVS(1463)*X(149)-JVS(1464)*X(150)-JVS(1465)*X(151)&
             &-JVS(1466)*X(152)
  X(154) = X(154)-JVS(1472)*X(54)-JVS(1473)*X(55)-JVS(1474)*X(57)-JVS(1475)*X(62)-JVS(1476)*X(63)-JVS(1477)*X(66)&
             &-JVS(1478)*X(68)-JVS(1479)*X(73)-JVS(1480)*X(81)-JVS(1481)*X(82)-JVS(1482)*X(83)-JVS(1483)*X(94)-JVS(1484)&
             &*X(95)-JVS(1485)*X(96)-JVS(1486)*X(99)-JVS(1487)*X(100)-JVS(1488)*X(101)-JVS(1489)*X(103)-JVS(1490)*X(104)&
             &-JVS(1491)*X(105)-JVS(1492)*X(107)-JVS(1493)*X(108)-JVS(1494)*X(111)-JVS(1495)*X(112)-JVS(1496)*X(113)&
             &-JVS(1497)*X(114)-JVS(1498)*X(115)-JVS(1499)*X(116)-JVS(1500)*X(117)-JVS(1501)*X(119)-JVS(1502)*X(120)&
             &-JVS(1503)*X(121)-JVS(1504)*X(122)-JVS(1505)*X(123)-JVS(1506)*X(124)-JVS(1507)*X(125)-JVS(1508)*X(126)&
             &-JVS(1509)*X(127)-JVS(1510)*X(128)-JVS(1511)*X(129)-JVS(1512)*X(130)-JVS(1513)*X(131)-JVS(1514)*X(132)&
             &-JVS(1515)*X(133)-JVS(1516)*X(134)-JVS(1517)*X(135)-JVS(1518)*X(136)-JVS(1519)*X(137)-JVS(1520)*X(138)&
             &-JVS(1521)*X(139)-JVS(1522)*X(140)-JVS(1523)*X(141)-JVS(1524)*X(142)-JVS(1525)*X(143)-JVS(1526)*X(144)&
             &-JVS(1527)*X(145)-JVS(1528)*X(146)-JVS(1529)*X(147)-JVS(1530)*X(148)-JVS(1531)*X(149)-JVS(1532)*X(150)&
             &-JVS(1533)*X(151)-JVS(1534)*X(152)-JVS(1535)*X(153)
  X(155) = X(155)-JVS(1540)*X(62)-JVS(1541)*X(63)-JVS(1542)*X(64)-JVS(1543)*X(68)-JVS(1544)*X(69)-JVS(1545)*X(73)&
             &-JVS(1546)*X(82)-JVS(1547)*X(85)-JVS(1548)*X(97)-JVS(1549)*X(98)-JVS(1550)*X(100)-JVS(1551)*X(104)-JVS(1552)&
             &*X(105)-JVS(1553)*X(108)-JVS(1554)*X(111)-JVS(1555)*X(113)-JVS(1556)*X(114)-JVS(1557)*X(116)-JVS(1558)*X(119)&
             &-JVS(1559)*X(120)-JVS(1560)*X(121)-JVS(1561)*X(122)-JVS(1562)*X(127)-JVS(1563)*X(128)-JVS(1564)*X(131)&
             &-JVS(1565)*X(132)-JVS(1566)*X(133)-JVS(1567)*X(134)-JVS(1568)*X(135)-JVS(1569)*X(136)-JVS(1570)*X(137)&
             &-JVS(1571)*X(138)-JVS(1572)*X(139)-JVS(1573)*X(140)-JVS(1574)*X(141)-JVS(1575)*X(142)-JVS(1576)*X(143)&
             &-JVS(1577)*X(144)-JVS(1578)*X(145)-JVS(1579)*X(146)-JVS(1580)*X(147)-JVS(1581)*X(148)-JVS(1582)*X(149)&
             &-JVS(1583)*X(150)-JVS(1584)*X(151)-JVS(1585)*X(152)-JVS(1586)*X(153)-JVS(1587)*X(154)
  X(156) = X(156)-JVS(1591)*X(59)-JVS(1592)*X(60)-JVS(1593)*X(61)-JVS(1594)*X(66)-JVS(1595)*X(99)-JVS(1596)*X(100)&
             &-JVS(1597)*X(101)-JVS(1598)*X(103)-JVS(1599)*X(107)-JVS(1600)*X(108)-JVS(1601)*X(111)-JVS(1602)*X(114)&
             &-JVS(1603)*X(115)-JVS(1604)*X(116)-JVS(1605)*X(120)-JVS(1606)*X(121)-JVS(1607)*X(122)-JVS(1608)*X(123)&
             &-JVS(1609)*X(124)-JVS(1610)*X(125)-JVS(1611)*X(126)-JVS(1612)*X(128)-JVS(1613)*X(129)-JVS(1614)*X(130)&
             &-JVS(1615)*X(131)-JVS(1616)*X(132)-JVS(1617)*X(133)-JVS(1618)*X(136)-JVS(1619)*X(137)-JVS(1620)*X(138)&
             &-JVS(1621)*X(139)-JVS(1622)*X(141)-JVS(1623)*X(142)-JVS(1624)*X(143)-JVS(1625)*X(144)-JVS(1626)*X(145)&
             &-JVS(1627)*X(146)-JVS(1628)*X(147)-JVS(1629)*X(148)-JVS(1630)*X(149)-JVS(1631)*X(150)-JVS(1632)*X(151)&
             &-JVS(1633)*X(152)-JVS(1634)*X(153)-JVS(1635)*X(154)-JVS(1636)*X(155)
  X(157) = X(157)-JVS(1639)*X(94)-JVS(1640)*X(107)-JVS(1641)*X(111)-JVS(1642)*X(122)-JVS(1643)*X(123)-JVS(1644)*X(128)&
             &-JVS(1645)*X(132)-JVS(1646)*X(133)-JVS(1647)*X(141)-JVS(1648)*X(142)-JVS(1649)*X(143)-JVS(1650)*X(145)&
             &-JVS(1651)*X(147)-JVS(1652)*X(148)-JVS(1653)*X(149)-JVS(1654)*X(150)-JVS(1655)*X(151)-JVS(1656)*X(152)&
             &-JVS(1657)*X(153)-JVS(1658)*X(154)-JVS(1659)*X(155)-JVS(1660)*X(156)
  X(157) = X(157)/JVS(1661)
  X(156) = (X(156)-JVS(1638)*X(157))/(JVS(1637))
  X(155) = (X(155)-JVS(1589)*X(156)-JVS(1590)*X(157))/(JVS(1588))
  X(154) = (X(154)-JVS(1537)*X(155)-JVS(1538)*X(156)-JVS(1539)*X(157))/(JVS(1536))
  X(153) = (X(153)-JVS(1468)*X(154)-JVS(1469)*X(155)-JVS(1470)*X(156)-JVS(1471)*X(157))/(JVS(1467))
  X(152) = (X(152)-JVS(1430)*X(153)-JVS(1431)*X(154)-JVS(1432)*X(155)-JVS(1433)*X(156)-JVS(1434)*X(157))/(JVS(1429))
  X(151) = (X(151)-JVS(1347)*X(152)-JVS(1348)*X(153)-JVS(1349)*X(154)-JVS(1350)*X(155)-JVS(1351)*X(156)-JVS(1352)&
             &*X(157))/(JVS(1346))
  X(150) = (X(150)-JVS(1300)*X(151)-JVS(1301)*X(152)-JVS(1302)*X(153)-JVS(1303)*X(154)-JVS(1304)*X(155)-JVS(1305)*X(156)&
             &-JVS(1306)*X(157))/(JVS(1299))
  X(149) = (X(149)-JVS(1198)*X(150)-JVS(1199)*X(151)-JVS(1200)*X(152)-JVS(1201)*X(153)-JVS(1202)*X(154)-JVS(1203)*X(155)&
             &-JVS(1204)*X(156)-JVS(1205)*X(157))/(JVS(1197))
  X(148) = (X(148)-JVS(1165)*X(149)-JVS(1166)*X(150)-JVS(1167)*X(151)-JVS(1168)*X(152)-JVS(1169)*X(153)-JVS(1170)*X(154)&
             &-JVS(1171)*X(155)-JVS(1172)*X(156)-JVS(1173)*X(157))/(JVS(1164))
  X(147) = (X(147)-JVS(1116)*X(148)-JVS(1117)*X(149)-JVS(1118)*X(150)-JVS(1119)*X(151)-JVS(1120)*X(152)-JVS(1121)*X(153)&
             &-JVS(1122)*X(154)-JVS(1123)*X(155)-JVS(1124)*X(156)-JVS(1125)*X(157))/(JVS(1115))
  X(146) = (X(146)-JVS(1086)*X(147)-JVS(1087)*X(148)-JVS(1088)*X(149)-JVS(1089)*X(150)-JVS(1090)*X(151)-JVS(1091)*X(152)&
             &-JVS(1092)*X(153)-JVS(1093)*X(154)-JVS(1094)*X(155)-JVS(1095)*X(156)-JVS(1096)*X(157))/(JVS(1085))
  X(145) = (X(145)-JVS(1047)*X(147)-JVS(1048)*X(148)-JVS(1049)*X(150)-JVS(1050)*X(151)-JVS(1051)*X(152)-JVS(1052)*X(155)&
             &-JVS(1053)*X(156)-JVS(1054)*X(157))/(JVS(1046))
  X(144) = (X(144)-JVS(1031)*X(147)-JVS(1032)*X(148)-JVS(1033)*X(150)-JVS(1034)*X(151)-JVS(1035)*X(152)-JVS(1036)*X(154)&
             &-JVS(1037)*X(155)-JVS(1038)*X(156))/(JVS(1030))
  X(143) = (X(143)-JVS(1017)*X(147)-JVS(1018)*X(148)-JVS(1019)*X(150)-JVS(1020)*X(151)-JVS(1021)*X(152)-JVS(1022)*X(155)&
             &-JVS(1023)*X(156))/(JVS(1016))
  X(142) = (X(142)-JVS(1005)*X(147)-JVS(1006)*X(148)-JVS(1007)*X(150)-JVS(1008)*X(151)-JVS(1009)*X(152)-JVS(1010)*X(155)&
             &-JVS(1011)*X(156))/(JVS(1004))
  X(141) = (X(141)-JVS(990)*X(142)-JVS(991)*X(145)-JVS(992)*X(147)-JVS(993)*X(148)-JVS(994)*X(150)-JVS(995)*X(151)&
             &-JVS(996)*X(152)-JVS(997)*X(155)-JVS(998)*X(156))/(JVS(989))
  X(140) = (X(140)-JVS(968)*X(141)-JVS(969)*X(142)-JVS(970)*X(143)-JVS(971)*X(144)-JVS(972)*X(147)-JVS(973)*X(148)&
             &-JVS(974)*X(149)-JVS(975)*X(150)-JVS(976)*X(151)-JVS(977)*X(152)-JVS(978)*X(153)-JVS(979)*X(154)-JVS(980)&
             &*X(155)-JVS(981)*X(156)-JVS(982)*X(157))/(JVS(967))
  X(139) = (X(139)-JVS(936)*X(142)-JVS(937)*X(143)-JVS(938)*X(147)-JVS(939)*X(148)-JVS(940)*X(150)-JVS(941)*X(151)&
             &-JVS(942)*X(152)-JVS(943)*X(155)-JVS(944)*X(156))/(JVS(935))
  X(138) = (X(138)-JVS(923)*X(147)-JVS(924)*X(148)-JVS(925)*X(150)-JVS(926)*X(151)-JVS(927)*X(152)-JVS(928)*X(155)&
             &-JVS(929)*X(156))/(JVS(922))
  X(137) = (X(137)-JVS(911)*X(138)-JVS(912)*X(142)-JVS(913)*X(147)-JVS(914)*X(148)-JVS(915)*X(150)-JVS(916)*X(151)&
             &-JVS(917)*X(152)-JVS(918)*X(155)-JVS(919)*X(156))/(JVS(910))
  X(136) = (X(136)-JVS(899)*X(138)-JVS(900)*X(142)-JVS(901)*X(147)-JVS(902)*X(148)-JVS(903)*X(150)-JVS(904)*X(151)&
             &-JVS(905)*X(152)-JVS(906)*X(155)-JVS(907)*X(156))/(JVS(898))
  X(135) = (X(135)-JVS(880)*X(136)-JVS(881)*X(141)-JVS(882)*X(142)-JVS(883)*X(145)-JVS(884)*X(146)-JVS(885)*X(147)&
             &-JVS(886)*X(148)-JVS(887)*X(149)-JVS(888)*X(150)-JVS(889)*X(151)-JVS(890)*X(152)-JVS(891)*X(153)-JVS(892)&
             &*X(154)-JVS(893)*X(155)-JVS(894)*X(156)-JVS(895)*X(157))/(JVS(879))
  X(134) = (X(134)-JVS(845)*X(135)-JVS(846)*X(136)-JVS(847)*X(137)-JVS(848)*X(138)-JVS(849)*X(139)-JVS(850)*X(140)&
             &-JVS(851)*X(141)-JVS(852)*X(142)-JVS(853)*X(143)-JVS(854)*X(144)-JVS(855)*X(145)-JVS(856)*X(146)-JVS(857)&
             &*X(147)-JVS(858)*X(148)-JVS(859)*X(149)-JVS(860)*X(150)-JVS(861)*X(151)-JVS(862)*X(152)-JVS(863)*X(153)&
             &-JVS(864)*X(154)-JVS(865)*X(155)-JVS(866)*X(156)-JVS(867)*X(157))/(JVS(844))
  X(133) = (X(133)-JVS(796)*X(142)-JVS(797)*X(147)-JVS(798)*X(148)-JVS(799)*X(150)-JVS(800)*X(151)-JVS(801)*X(155)&
             &-JVS(802)*X(156))/(JVS(795))
  X(132) = (X(132)-JVS(785)*X(141)-JVS(786)*X(148)-JVS(787)*X(149)-JVS(788)*X(150)-JVS(789)*X(151)-JVS(790)*X(152)&
             &-JVS(791)*X(153)-JVS(792)*X(155)-JVS(793)*X(156)-JVS(794)*X(157))/(JVS(784))
  X(131) = (X(131)-JVS(770)*X(138)-JVS(771)*X(139)-JVS(772)*X(143)-JVS(773)*X(147)-JVS(774)*X(148)-JVS(775)*X(150)&
             &-JVS(776)*X(151)-JVS(777)*X(152)-JVS(778)*X(155)-JVS(779)*X(156))/(JVS(769))
  X(130) = (X(130)-JVS(755)*X(133)-JVS(756)*X(136)-JVS(757)*X(146)-JVS(758)*X(148)-JVS(759)*X(149)-JVS(760)*X(150)&
             &-JVS(761)*X(151)-JVS(762)*X(152)-JVS(763)*X(154)-JVS(764)*X(155)-JVS(765)*X(156))/(JVS(754))
  X(129) = (X(129)-JVS(739)*X(130)-JVS(740)*X(141)-JVS(741)*X(146)-JVS(742)*X(148)-JVS(743)*X(149)-JVS(744)*X(150)&
             &-JVS(745)*X(151)-JVS(746)*X(152)-JVS(747)*X(153)-JVS(748)*X(155)-JVS(749)*X(156)-JVS(750)*X(157))/(JVS(738))
  X(128) = (X(128)-JVS(729)*X(148)-JVS(730)*X(149)-JVS(731)*X(150)-JVS(732)*X(151)-JVS(733)*X(152)-JVS(734)*X(155)&
             &-JVS(735)*X(156))/(JVS(728))
  X(127) = (X(127)-JVS(710)*X(128)-JVS(711)*X(131)-JVS(712)*X(132)-JVS(713)*X(136)-JVS(714)*X(137)-JVS(715)*X(138)&
             &-JVS(716)*X(139)-JVS(717)*X(140)-JVS(718)*X(142)-JVS(719)*X(143)-JVS(720)*X(147)-JVS(721)*X(148)-JVS(722)&
             &*X(150)-JVS(723)*X(151)-JVS(724)*X(152)-JVS(725)*X(154)-JVS(726)*X(155)-JVS(727)*X(156))/(JVS(709))
  X(126) = (X(126)-JVS(687)*X(136)-JVS(688)*X(147)-JVS(689)*X(148)-JVS(690)*X(150)-JVS(691)*X(151)-JVS(692)*X(152)&
             &-JVS(693)*X(155)-JVS(694)*X(156))/(JVS(686))
  X(125) = (X(125)-JVS(676)*X(141)-JVS(677)*X(148)-JVS(678)*X(150)-JVS(679)*X(151)-JVS(680)*X(152)-JVS(681)*X(153)&
             &-JVS(682)*X(156)-JVS(683)*X(157))/(JVS(675))
  X(124) = (X(124)-JVS(665)*X(136)-JVS(666)*X(147)-JVS(667)*X(148)-JVS(668)*X(150)-JVS(669)*X(151)-JVS(670)*X(152)&
             &-JVS(671)*X(155)-JVS(672)*X(156))/(JVS(664))
  X(123) = (X(123)-JVS(656)*X(143)-JVS(657)*X(148)-JVS(658)*X(151)-JVS(659)*X(152)-JVS(660)*X(155)-JVS(661)*X(156))&
             &/(JVS(655))
  X(122) = (X(122)-JVS(649)*X(133)-JVS(650)*X(150)-JVS(651)*X(152)-JVS(652)*X(154)-JVS(653)*X(155)-JVS(654)*X(156))&
             &/(JVS(648))
  X(121) = (X(121)-JVS(642)*X(143)-JVS(643)*X(148)-JVS(644)*X(150)-JVS(645)*X(151)-JVS(646)*X(152)-JVS(647)*X(156))&
             &/(JVS(641))
  X(120) = (X(120)-JVS(634)*X(136)-JVS(635)*X(148)-JVS(636)*X(150)-JVS(637)*X(151)-JVS(638)*X(152)-JVS(639)*X(156))&
             &/(JVS(633))
  X(119) = (X(119)-JVS(618)*X(120)-JVS(619)*X(131)-JVS(620)*X(133)-JVS(621)*X(137)-JVS(622)*X(138)-JVS(623)*X(139)&
             &-JVS(624)*X(142)-JVS(625)*X(147)-JVS(626)*X(148)-JVS(627)*X(150)-JVS(628)*X(151)-JVS(629)*X(152)-JVS(630)&
             &*X(155)-JVS(631)*X(156))/(JVS(617))
  X(118) = (X(118)-JVS(601)*X(138)-JVS(602)*X(142)-JVS(603)*X(147)-JVS(604)*X(148)-JVS(605)*X(150)-JVS(606)*X(151)&
             &-JVS(607)*X(152)-JVS(608)*X(155)-JVS(609)*X(156))/(JVS(600))
  X(117) = (X(117)-JVS(586)*X(123)-JVS(587)*X(124)-JVS(588)*X(138)-JVS(589)*X(139)-JVS(590)*X(142)-JVS(591)*X(147)&
             &-JVS(592)*X(148)-JVS(593)*X(150)-JVS(594)*X(151)-JVS(595)*X(152)-JVS(596)*X(156))/(JVS(585))
  X(116) = (X(116)-JVS(577)*X(144)-JVS(578)*X(148)-JVS(579)*X(150)-JVS(580)*X(151)-JVS(581)*X(152)-JVS(582)*X(156))&
             &/(JVS(576))
  X(115) = (X(115)-JVS(567)*X(141)-JVS(568)*X(148)-JVS(569)*X(150)-JVS(570)*X(151)-JVS(571)*X(152)-JVS(572)*X(153)&
             &-JVS(573)*X(156)-JVS(574)*X(157))/(JVS(566))
  X(114) = (X(114)-JVS(556)*X(121)-JVS(557)*X(131)-JVS(558)*X(138)-JVS(559)*X(139)-JVS(560)*X(147)-JVS(561)*X(150)&
             &-JVS(562)*X(152)-JVS(563)*X(156))/(JVS(555))
  X(113) = (X(113)-JVS(533)*X(116)-JVS(534)*X(119)-JVS(535)*X(127)-JVS(536)*X(131)-JVS(537)*X(133)-JVS(538)*X(134)&
             &-JVS(539)*X(135)-JVS(540)*X(137)-JVS(541)*X(138)-JVS(542)*X(139)-JVS(543)*X(143)-JVS(544)*X(146)-JVS(545)&
             &*X(147)-JVS(546)*X(148)-JVS(547)*X(149)-JVS(548)*X(150)-JVS(549)*X(152)-JVS(550)*X(153)-JVS(551)*X(154)&
             &-JVS(552)*X(155)-JVS(553)*X(156))/(JVS(532))
  X(112) = (X(112)-JVS(518)*X(138)-JVS(519)*X(142)-JVS(520)*X(147)-JVS(521)*X(150)-JVS(522)*X(151)-JVS(523)*X(156))&
             &/(JVS(517))
  X(111) = (X(111)-JVS(512)*X(133)-JVS(513)*X(151)-JVS(514)*X(152)-JVS(515)*X(155)-JVS(516)*X(156))/(JVS(511))
  X(110) = (X(110)-JVS(498)*X(112)-JVS(499)*X(118)-JVS(500)*X(120)-JVS(501)*X(131)-JVS(502)*X(137)-JVS(503)*X(138)&
             &-JVS(504)*X(139)-JVS(505)*X(147)-JVS(506)*X(148)-JVS(507)*X(150)-JVS(508)*X(151)-JVS(509)*X(152)-JVS(510)&
             &*X(156))/(JVS(497))
  X(109) = (X(109)-JVS(462)*X(110)-JVS(463)*X(114)-JVS(464)*X(117)-JVS(465)*X(118)-JVS(466)*X(119)-JVS(467)*X(121)&
             &-JVS(468)*X(122)-JVS(469)*X(123)-JVS(470)*X(125)-JVS(471)*X(127)-JVS(472)*X(134)-JVS(473)*X(135)-JVS(474)&
             &*X(136)-JVS(475)*X(137)-JVS(476)*X(138)-JVS(477)*X(140)-JVS(478)*X(141)-JVS(479)*X(143)-JVS(480)*X(144)&
             &-JVS(481)*X(146)-JVS(482)*X(147)-JVS(483)*X(148)-JVS(484)*X(150)-JVS(485)*X(151)-JVS(486)*X(152)-JVS(487)&
             &*X(153)-JVS(488)*X(154)-JVS(489)*X(155)-JVS(490)*X(156)-JVS(491)*X(157))/(JVS(461))
  X(108) = (X(108)-JVS(444)*X(147)-JVS(445)*X(150)-JVS(446)*X(152)-JVS(447)*X(153)-JVS(448)*X(154)-JVS(449)*X(155)&
             &-JVS(450)*X(156))/(JVS(443))
  X(107) = (X(107)-JVS(437)*X(133)-JVS(438)*X(150)-JVS(439)*X(151)-JVS(440)*X(152)-JVS(441)*X(156))/(JVS(436))
  X(106) = (X(106)-JVS(427)*X(125)-JVS(428)*X(141)-JVS(429)*X(148)-JVS(430)*X(150)-JVS(431)*X(151)-JVS(432)*X(152)&
             &-JVS(433)*X(153)-JVS(434)*X(156)-JVS(435)*X(157))/(JVS(426))
  X(105) = (X(105)-JVS(418)*X(120)-JVS(419)*X(131)-JVS(420)*X(138)-JVS(421)*X(147)-JVS(422)*X(150)-JVS(423)*X(152)&
             &-JVS(424)*X(156))/(JVS(417))
  X(104) = (X(104)-JVS(411)*X(122)-JVS(412)*X(144)-JVS(413)*X(147)-JVS(414)*X(150)-JVS(415)*X(154))/(JVS(410))
  X(103) = (X(103)-JVS(406)*X(138)-JVS(407)*X(148)-JVS(408)*X(152)-JVS(409)*X(156))/(JVS(405))
  X(102) = (X(102)-JVS(391)*X(115)-JVS(392)*X(116)-JVS(393)*X(120)-JVS(394)*X(121)-JVS(395)*X(124)-JVS(396)*X(125)&
             &-JVS(397)*X(126)-JVS(398)*X(128)-JVS(399)*X(137)-JVS(400)*X(138)-JVS(401)*X(141)-JVS(402)*X(145)-JVS(403)&
             &*X(150)-JVS(404)*X(151))/(JVS(390))
  X(101) = (X(101)-JVS(384)*X(137)-JVS(385)*X(147)-JVS(386)*X(148)-JVS(387)*X(150)-JVS(388)*X(152)-JVS(389)*X(156))&
             &/(JVS(383))
  X(100) = (X(100)-JVS(377)*X(114)-JVS(378)*X(150)-JVS(379)*X(152)-JVS(380)*X(154)-JVS(381)*X(156))/(JVS(376))
  X(99) = (X(99)-JVS(371)*X(138)-JVS(372)*X(150)-JVS(373)*X(152)-JVS(374)*X(156))/(JVS(370))
  X(98) = (X(98)-JVS(364)*X(136)-JVS(365)*X(147)-JVS(366)*X(150)-JVS(367)*X(155))/(JVS(363))
  X(97) = (X(97)-JVS(359)*X(147)-JVS(360)*X(150)-JVS(361)*X(155))/(JVS(358))
  X(96) = (X(96)-JVS(354)*X(138)-JVS(355)*X(147)-JVS(356)*X(150)-JVS(357)*X(156))/(JVS(353))
  X(95) = (X(95)-JVS(348)*X(123)-JVS(349)*X(131)-JVS(350)*X(139)-JVS(351)*X(150)-JVS(352)*X(152))/(JVS(347))
  X(94) = (X(94)-JVS(342)*X(107)-JVS(343)*X(111)-JVS(344)*X(142)-JVS(345)*X(150)-JVS(346)*X(152))/(JVS(341))
  X(93) = (X(93)-JVS(336)*X(137)-JVS(337)*X(147)-JVS(338)*X(148)-JVS(339)*X(150)-JVS(340)*X(156))/(JVS(335))
  X(92) = (X(92)-JVS(330)*X(137)-JVS(331)*X(148)-JVS(332)*X(150)-JVS(333)*X(152)-JVS(334)*X(156))/(JVS(329))
  X(91) = (X(91)-JVS(325)*X(137)-JVS(326)*X(150)-JVS(327)*X(152))/(JVS(324))
  X(90) = (X(90)-JVS(321)*X(138)-JVS(322)*X(150)-JVS(323)*X(152))/(JVS(320))
  X(89) = (X(89)-JVS(317)*X(120)-JVS(318)*X(150)-JVS(319)*X(152))/(JVS(316))
  X(88) = (X(88)-JVS(311)*X(101)-JVS(312)*X(103)-JVS(313)*X(141)-JVS(314)*X(150)-JVS(315)*X(152))/(JVS(310))
  X(87) = (X(87)-JVS(306)*X(128)-JVS(307)*X(132)-JVS(308)*X(150)-JVS(309)*X(152))/(JVS(305))
  X(86) = (X(86)-JVS(302)*X(121)-JVS(303)*X(150)-JVS(304)*X(152))/(JVS(301))
  X(85) = (X(85)-JVS(298)*X(150)-JVS(299)*X(153)-JVS(300)*X(155))/(JVS(297))
  X(84) = (X(84)-JVS(294)*X(126)-JVS(295)*X(150)-JVS(296)*X(152))/(JVS(293))
  X(83) = (X(83)-JVS(287)*X(112)-JVS(288)*X(139)-JVS(289)*X(147)-JVS(290)*X(150)-JVS(291)*X(152)-JVS(292)*X(156))&
            &/(JVS(286))
  X(82) = (X(82)-JVS(283)*X(108)-JVS(284)*X(153)-JVS(285)*X(154))/(JVS(282))
  X(81) = (X(81)-JVS(279)*X(124)-JVS(280)*X(150)-JVS(281)*X(152))/(JVS(278))
  X(80) = (X(80)-JVS(274)*X(122)-JVS(275)*X(130)-JVS(276)*X(150)-JVS(277)*X(152))/(JVS(273))
  X(79) = (X(79)-JVS(270)*X(144)-JVS(271)*X(150)-JVS(272)*X(152))/(JVS(269))
  X(78) = (X(78)-JVS(262)*X(85)-JVS(263)*X(106)-JVS(264)*X(134)-JVS(265)*X(135)-JVS(266)*X(150)-JVS(267)*X(152)-JVS(268)&
            &*X(153))/(JVS(261))
  X(77) = (X(77)-JVS(254)*X(78)-JVS(255)*X(82)-JVS(256)*X(108)-JVS(257)*X(150)-JVS(258)*X(152)-JVS(259)*X(153))&
            &/(JVS(253))
  X(76) = (X(76)-JVS(247)*X(77)-JVS(248)*X(82)-JVS(249)*X(108)-JVS(250)*X(150)-JVS(251)*X(152))/(JVS(246))
  X(75) = (X(75)-JVS(243)*X(150)-JVS(244)*X(151)-JVS(245)*X(152))/(JVS(242))
  X(74) = (X(74)-JVS(239)*X(125)-JVS(240)*X(150)-JVS(241)*X(152))/(JVS(238))
  X(73) = (X(73)-JVS(235)*X(150)-JVS(236)*X(152)-JVS(237)*X(154))/(JVS(234))
  X(72) = (X(72)-JVS(231)*X(115)-JVS(232)*X(150)-JVS(233)*X(152))/(JVS(230))
  X(71) = (X(71)-JVS(227)*X(129)-JVS(228)*X(150)-JVS(229)*X(152))/(JVS(226))
  X(70) = (X(70)-JVS(224)*X(150)-JVS(225)*X(153))/(JVS(223))
  X(69) = (X(69)-JVS(221)*X(150)-JVS(222)*X(155))/(JVS(220))
  X(68) = (X(68)-JVS(218)*X(151)-JVS(219)*X(154))/(JVS(217))
  X(67) = (X(67)-JVS(214)*X(148)-JVS(215)*X(150)-JVS(216)*X(152))/(JVS(213))
  X(66) = (X(66)-JVS(210)*X(150)-JVS(211)*X(154)-JVS(212)*X(156))/(JVS(209))
  X(65) = (X(65)-JVS(205)*X(97)-JVS(206)*X(147)-JVS(207)*X(150)-JVS(208)*X(152))/(JVS(204))
  X(64) = (X(64)-JVS(202)*X(150)-JVS(203)*X(155))/(JVS(201))
  X(63) = (X(63)-JVS(199)*X(148)-JVS(200)*X(154))/(JVS(198))
  X(62) = (X(62)-JVS(196)*X(154)-JVS(197)*X(155))/(JVS(195))
  X(61) = (X(61)-JVS(192)*X(150)-JVS(193)*X(152)-JVS(194)*X(156))/(JVS(191))
  X(60) = (X(60)-JVS(187)*X(150)-JVS(188)*X(152)-JVS(189)*X(156))/(JVS(186))
  X(59) = (X(59)-JVS(182)*X(150)-JVS(183)*X(152)-JVS(184)*X(156))/(JVS(181))
  X(58) = (X(58)-JVS(177)*X(69)-JVS(178)*X(150)-JVS(179)*X(155))/(JVS(176))
  X(57) = (X(57)-JVS(174)*X(100)-JVS(175)*X(154))/(JVS(173))
  X(56) = (X(56)-JVS(171)*X(90)-JVS(172)*X(150))/(JVS(170))
  X(55) = (X(55)-JVS(168)*X(153)-JVS(169)*X(154))/(JVS(167))
  X(54) = (X(54)-JVS(165)*X(130)-JVS(166)*X(154))/(JVS(164))
  X(53) = (X(53)-JVS(163)*X(150))/(JVS(162))
  X(52) = (X(52)-JVS(161)*X(150))/(JVS(160))
  X(51) = (X(51)-JVS(159)*X(150))/(JVS(158))
  X(50) = (X(50)-JVS(157)*X(150))/(JVS(156))
  X(49) = (X(49)-JVS(155)*X(150))/(JVS(154))
  X(48) = (X(48)-JVS(153)*X(150))/(JVS(152))
  X(47) = (X(47)-JVS(125)*X(54)-JVS(126)*X(56)-JVS(127)*X(57)-JVS(128)*X(62)-JVS(129)*X(63)-JVS(130)*X(65)-JVS(131)&
            &*X(76)-JVS(132)*X(77)-JVS(133)*X(78)-JVS(134)*X(82)-JVS(135)*X(90)-JVS(136)*X(96)-JVS(137)*X(104)-JVS(138)&
            &*X(105)-JVS(139)*X(106)-JVS(140)*X(112)-JVS(141)*X(113)-JVS(142)*X(114)-JVS(143)*X(117)-JVS(144)*X(134)&
            &-JVS(145)*X(135)-JVS(146)*X(136)-JVS(147)*X(140)-JVS(148)*X(143)-JVS(149)*X(147)-JVS(150)*X(154)-JVS(151)&
            &*X(157))/(JVS(124))
  X(46) = (X(46)-JVS(105)*X(83)-JVS(106)*X(86)-JVS(107)*X(93)-JVS(108)*X(99)-JVS(109)*X(100)-JVS(110)*X(101)-JVS(111)&
            &*X(109)-JVS(112)*X(110)-JVS(113)*X(116)-JVS(114)*X(123)-JVS(115)*X(138)-JVS(116)*X(140)-JVS(117)*X(144)&
            &-JVS(118)*X(147)-JVS(119)*X(148)-JVS(120)*X(150)-JVS(121)*X(151)-JVS(122)*X(152)-JVS(123)*X(156))/(JVS(104))
  X(45) = X(45)/JVS(103)
  X(44) = X(44)/JVS(102)
  X(43) = (X(43)-JVS(101)*X(117))/(JVS(100))
  X(42) = (X(42)-JVS(99)*X(105))/(JVS(98))
  X(41) = (X(41)-JVS(97)*X(114))/(JVS(96))
  X(40) = (X(40)-JVS(95)*X(56))/(JVS(94))
  X(39) = (X(39)-JVS(93)*X(90))/(JVS(92))
  X(38) = (X(38)-JVS(91)*X(96))/(JVS(90))
  X(37) = (X(37)-JVS(89)*X(112))/(JVS(88))
  X(36) = (X(36)-JVS(86)*X(105)-JVS(87)*X(150))/(JVS(85))
  X(35) = (X(35)-JVS(84)*X(77))/(JVS(83))
  X(34) = (X(34)-JVS(82)*X(82))/(JVS(81))
  X(33) = (X(33)-JVS(80)*X(78))/(JVS(79))
  X(32) = (X(32)-JVS(78)*X(76))/(JVS(77))
  X(31) = (X(31)-JVS(76)*X(150))/(JVS(75))
  X(30) = (X(30)-JVS(71)*X(31)-JVS(72)*X(150)-JVS(73)*X(152)-JVS(74)*X(156))/(JVS(70))
  X(29) = (X(29)-JVS(68)*X(30)-JVS(69)*X(156))/(JVS(67))
  X(28) = (X(28)-JVS(65)*X(30)-JVS(66)*X(152))/(JVS(64))
  X(27) = (X(27)-JVS(62)*X(61)-JVS(63)*X(156))/(JVS(61))
  X(26) = (X(26)-JVS(59)*X(61)-JVS(60)*X(152))/(JVS(58))
  X(25) = (X(25)-JVS(56)*X(60)-JVS(57)*X(156))/(JVS(55))
  X(24) = (X(24)-JVS(53)*X(60)-JVS(54)*X(152))/(JVS(52))
  X(23) = (X(23)-JVS(50)*X(59)-JVS(51)*X(156))/(JVS(49))
  X(22) = (X(22)-JVS(47)*X(59)-JVS(48)*X(152))/(JVS(46))
  X(21) = (X(21)-JVS(44)*X(97)-JVS(45)*X(155))/(JVS(43))
  X(20) = (X(20)-JVS(41)*X(97)-JVS(42)*X(150))/(JVS(40))
  X(19) = (X(19)-JVS(38)*X(69)-JVS(39)*X(150))/(JVS(37))
  X(18) = (X(18)-JVS(35)*X(58)-JVS(36)*X(150))/(JVS(34))
  X(17) = (X(17)-JVS(33)*X(135))/(JVS(32))
  X(16) = (X(16)-JVS(31)*X(140))/(JVS(30))
  X(15) = (X(15)-JVS(29)*X(143))/(JVS(28))
  X(14) = (X(14)-JVS(27)*X(136))/(JVS(26))
  X(13) = (X(13)-JVS(25)*X(157))/(JVS(24))
  X(12) = (X(12)-JVS(23)*X(54))/(JVS(22))
  X(11) = (X(11)-JVS(21)*X(57))/(JVS(20))
  X(10) = (X(10)-JVS(19)*X(104))/(JVS(18))
  X(9) = (X(9)-JVS(17)*X(63))/(JVS(16))
  X(8) = (X(8)-JVS(15)*X(147))/(JVS(14))
  X(7) = (X(7)-JVS(13)*X(154))/(JVS(12))
  X(6) = (X(6)-JVS(11)*X(62))/(JVS(10))
  X(5) = (X(5)-JVS(9)*X(113))/(JVS(8))
  X(4) = (X(4)-JVS(7)*X(65))/(JVS(6))
  X(3) = (X(3)-JVS(5)*X(134))/(JVS(4))
  X(2) = (X(2)-JVS(3)*X(106))/(JVS(2))
  X(1) = X(1)/JVS(1)
      
END SUBROUTINE KppSolve

! End of KppSolve function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! KppSolveTR - sparse, transposed back substitution
!   Arguments :
!      JVS       - sparse Jacobian of variables
!      X         - Vector for variables
!      XX        - Vector for output variables
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SUBROUTINE KppSolveTR ( JVS, X, XX )

! JVS - sparse Jacobian of variables
  REAL(kind=dp) :: JVS(LU_NONZERO)
! X - Vector for variables
  REAL(kind=dp) :: X(NVAR)
! XX - Vector for output variables
  REAL(kind=dp) :: XX(NVAR)

  XX(1) = X(1)/JVS(1)
  XX(2) = X(2)/JVS(2)
  XX(3) = X(3)/JVS(4)
  XX(4) = X(4)/JVS(6)
  XX(5) = X(5)/JVS(8)
  XX(6) = X(6)/JVS(10)
  XX(7) = X(7)/JVS(12)
  XX(8) = X(8)/JVS(14)
  XX(9) = X(9)/JVS(16)
  XX(10) = X(10)/JVS(18)
  XX(11) = X(11)/JVS(20)
  XX(12) = X(12)/JVS(22)
  XX(13) = X(13)/JVS(24)
  XX(14) = X(14)/JVS(26)
  XX(15) = X(15)/JVS(28)
  XX(16) = X(16)/JVS(30)
  XX(17) = X(17)/JVS(32)
  XX(18) = X(18)/JVS(34)
  XX(19) = X(19)/JVS(37)
  XX(20) = X(20)/JVS(40)
  XX(21) = X(21)/JVS(43)
  XX(22) = X(22)/JVS(46)
  XX(23) = X(23)/JVS(49)
  XX(24) = X(24)/JVS(52)
  XX(25) = X(25)/JVS(55)
  XX(26) = X(26)/JVS(58)
  XX(27) = X(27)/JVS(61)
  XX(28) = X(28)/JVS(64)
  XX(29) = X(29)/JVS(67)
  XX(30) = (X(30)-JVS(65)*XX(28)-JVS(68)*XX(29))/(JVS(70))
  XX(31) = (X(31)-JVS(71)*XX(30))/(JVS(75))
  XX(32) = X(32)/JVS(77)
  XX(33) = X(33)/JVS(79)
  XX(34) = X(34)/JVS(81)
  XX(35) = X(35)/JVS(83)
  XX(36) = X(36)/JVS(85)
  XX(37) = X(37)/JVS(88)
  XX(38) = X(38)/JVS(90)
  XX(39) = X(39)/JVS(92)
  XX(40) = X(40)/JVS(94)
  XX(41) = X(41)/JVS(96)
  XX(42) = X(42)/JVS(98)
  XX(43) = X(43)/JVS(100)
  XX(44) = X(44)/JVS(102)
  XX(45) = X(45)/JVS(103)
  XX(46) = X(46)/JVS(104)
  XX(47) = X(47)/JVS(124)
  XX(48) = X(48)/JVS(152)
  XX(49) = X(49)/JVS(154)
  XX(50) = X(50)/JVS(156)
  XX(51) = X(51)/JVS(158)
  XX(52) = X(52)/JVS(160)
  XX(53) = X(53)/JVS(162)
  XX(54) = (X(54)-JVS(23)*XX(12)-JVS(125)*XX(47))/(JVS(164))
  XX(55) = X(55)/JVS(167)
  XX(56) = (X(56)-JVS(95)*XX(40)-JVS(126)*XX(47))/(JVS(170))
  XX(57) = (X(57)-JVS(21)*XX(11)-JVS(127)*XX(47))/(JVS(173))
  XX(58) = (X(58)-JVS(35)*XX(18))/(JVS(176))
  XX(59) = (X(59)-JVS(47)*XX(22)-JVS(50)*XX(23))/(JVS(181))
  XX(60) = (X(60)-JVS(53)*XX(24)-JVS(56)*XX(25))/(JVS(186))
  XX(61) = (X(61)-JVS(59)*XX(26)-JVS(62)*XX(27))/(JVS(191))
  XX(62) = (X(62)-JVS(11)*XX(6)-JVS(128)*XX(47))/(JVS(195))
  XX(63) = (X(63)-JVS(17)*XX(9)-JVS(129)*XX(47))/(JVS(198))
  XX(64) = X(64)/JVS(201)
  XX(65) = (X(65)-JVS(7)*XX(4)-JVS(130)*XX(47))/(JVS(204))
  XX(66) = X(66)/JVS(209)
  XX(67) = X(67)/JVS(213)
  XX(68) = X(68)/JVS(217)
  XX(69) = (X(69)-JVS(38)*XX(19)-JVS(177)*XX(58))/(JVS(220))
  XX(70) = X(70)/JVS(223)
  XX(71) = X(71)/JVS(226)
  XX(72) = X(72)/JVS(230)
  XX(73) = X(73)/JVS(234)
  XX(74) = X(74)/JVS(238)
  XX(75) = X(75)/JVS(242)
  XX(76) = (X(76)-JVS(78)*XX(32)-JVS(131)*XX(47))/(JVS(246))
  XX(77) = (X(77)-JVS(84)*XX(35)-JVS(132)*XX(47)-JVS(247)*XX(76))/(JVS(253))
  XX(78) = (X(78)-JVS(80)*XX(33)-JVS(133)*XX(47)-JVS(254)*XX(77))/(JVS(261))
  XX(79) = X(79)/JVS(269)
  XX(80) = X(80)/JVS(273)
  XX(81) = X(81)/JVS(278)
  XX(82) = (X(82)-JVS(82)*XX(34)-JVS(134)*XX(47)-JVS(248)*XX(76)-JVS(255)*XX(77))/(JVS(282))
  XX(83) = (X(83)-JVS(105)*XX(46))/(JVS(286))
  XX(84) = X(84)/JVS(293)
  XX(85) = (X(85)-JVS(262)*XX(78))/(JVS(297))
  XX(86) = (X(86)-JVS(106)*XX(46))/(JVS(301))
  XX(87) = X(87)/JVS(305)
  XX(88) = X(88)/JVS(310)
  XX(89) = X(89)/JVS(316)
  XX(90) = (X(90)-JVS(93)*XX(39)-JVS(135)*XX(47)-JVS(171)*XX(56))/(JVS(320))
  XX(91) = X(91)/JVS(324)
  XX(92) = X(92)/JVS(329)
  XX(93) = (X(93)-JVS(107)*XX(46))/(JVS(335))
  XX(94) = X(94)/JVS(341)
  XX(95) = X(95)/JVS(347)
  XX(96) = (X(96)-JVS(91)*XX(38)-JVS(136)*XX(47))/(JVS(353))
  XX(97) = (X(97)-JVS(41)*XX(20)-JVS(44)*XX(21)-JVS(205)*XX(65))/(JVS(358))
  XX(98) = X(98)/JVS(363)
  XX(99) = (X(99)-JVS(108)*XX(46))/(JVS(370))
  XX(100) = (X(100)-JVS(109)*XX(46)-JVS(174)*XX(57))/(JVS(376))
  XX(101) = (X(101)-JVS(110)*XX(46)-JVS(311)*XX(88))/(JVS(383))
  XX(102) = X(102)/JVS(390)
  XX(103) = (X(103)-JVS(312)*XX(88))/(JVS(405))
  XX(104) = (X(104)-JVS(19)*XX(10)-JVS(137)*XX(47))/(JVS(410))
  XX(105) = (X(105)-JVS(86)*XX(36)-JVS(99)*XX(42)-JVS(138)*XX(47))/(JVS(417))
  XX(106) = (X(106)-JVS(3)*XX(2)-JVS(139)*XX(47)-JVS(263)*XX(78))/(JVS(426))
  XX(107) = (X(107)-JVS(342)*XX(94))/(JVS(436))
  XX(108) = (X(108)-JVS(249)*XX(76)-JVS(256)*XX(77)-JVS(283)*XX(82))/(JVS(443))
  XX(109) = (X(109)-JVS(111)*XX(46))/(JVS(461))
  XX(110) = (X(110)-JVS(112)*XX(46)-JVS(462)*XX(109))/(JVS(497))
  XX(111) = (X(111)-JVS(343)*XX(94))/(JVS(511))
  XX(112) = (X(112)-JVS(89)*XX(37)-JVS(140)*XX(47)-JVS(287)*XX(83)-JVS(498)*XX(110))/(JVS(517))
  XX(113) = (X(113)-JVS(9)*XX(5)-JVS(141)*XX(47))/(JVS(532))
  XX(114) = (X(114)-JVS(97)*XX(41)-JVS(142)*XX(47)-JVS(377)*XX(100)-JVS(463)*XX(109))/(JVS(555))
  XX(115) = (X(115)-JVS(231)*XX(72)-JVS(391)*XX(102))/(JVS(566))
  XX(116) = (X(116)-JVS(113)*XX(46)-JVS(392)*XX(102)-JVS(533)*XX(113))/(JVS(576))
  XX(117) = (X(117)-JVS(101)*XX(43)-JVS(143)*XX(47)-JVS(464)*XX(109))/(JVS(585))
  XX(118) = (X(118)-JVS(465)*XX(109)-JVS(499)*XX(110))/(JVS(600))
  XX(119) = (X(119)-JVS(466)*XX(109)-JVS(534)*XX(113))/(JVS(617))
  XX(120) = (X(120)-JVS(317)*XX(89)-JVS(393)*XX(102)-JVS(418)*XX(105)-JVS(500)*XX(110)-JVS(618)*XX(119))/(JVS(633))
  XX(121) = (X(121)-JVS(302)*XX(86)-JVS(394)*XX(102)-JVS(467)*XX(109)-JVS(556)*XX(114))/(JVS(641))
  XX(122) = (X(122)-JVS(274)*XX(80)-JVS(411)*XX(104)-JVS(468)*XX(109))/(JVS(648))
  XX(123) = (X(123)-JVS(114)*XX(46)-JVS(348)*XX(95)-JVS(469)*XX(109)-JVS(586)*XX(117))/(JVS(655))
  XX(124) = (X(124)-JVS(279)*XX(81)-JVS(395)*XX(102)-JVS(587)*XX(117))/(JVS(664))
  XX(125) = (X(125)-JVS(239)*XX(74)-JVS(396)*XX(102)-JVS(427)*XX(106)-JVS(470)*XX(109))/(JVS(675))
  XX(126) = (X(126)-JVS(294)*XX(84)-JVS(397)*XX(102))/(JVS(686))
  XX(127) = (X(127)-JVS(471)*XX(109)-JVS(535)*XX(113))/(JVS(709))
  XX(128) = (X(128)-JVS(306)*XX(87)-JVS(398)*XX(102)-JVS(710)*XX(127))/(JVS(728))
  XX(129) = (X(129)-JVS(227)*XX(71))/(JVS(738))
  XX(130) = (X(130)-JVS(165)*XX(54)-JVS(275)*XX(80)-JVS(739)*XX(129))/(JVS(754))
  XX(131) = (X(131)-JVS(349)*XX(95)-JVS(419)*XX(105)-JVS(501)*XX(110)-JVS(536)*XX(113)-JVS(557)*XX(114)-JVS(619)*XX(119)&
              &-JVS(711)*XX(127))/(JVS(769))
  XX(132) = (X(132)-JVS(307)*XX(87)-JVS(712)*XX(127))/(JVS(784))
  XX(133) = (X(133)-JVS(437)*XX(107)-JVS(512)*XX(111)-JVS(537)*XX(113)-JVS(620)*XX(119)-JVS(649)*XX(122)-JVS(755)&
              &*XX(130))/(JVS(795))
  XX(134) = (X(134)-JVS(5)*XX(3)-JVS(144)*XX(47)-JVS(264)*XX(78)-JVS(472)*XX(109)-JVS(538)*XX(113))/(JVS(844))
  XX(135) = (X(135)-JVS(33)*XX(17)-JVS(145)*XX(47)-JVS(265)*XX(78)-JVS(473)*XX(109)-JVS(539)*XX(113)-JVS(845)*XX(134))&
              &/(JVS(879))
  XX(136) = (X(136)-JVS(27)*XX(14)-JVS(146)*XX(47)-JVS(364)*XX(98)-JVS(474)*XX(109)-JVS(634)*XX(120)-JVS(665)*XX(124)&
              &-JVS(687)*XX(126)-JVS(713)*XX(127)-JVS(756)*XX(130)-JVS(846)*XX(134)-JVS(880)*XX(135))/(JVS(898))
  XX(137) = (X(137)-JVS(325)*XX(91)-JVS(330)*XX(92)-JVS(336)*XX(93)-JVS(384)*XX(101)-JVS(399)*XX(102)-JVS(475)*XX(109)&
              &-JVS(502)*XX(110)-JVS(540)*XX(113)-JVS(621)*XX(119)-JVS(714)*XX(127)-JVS(847)*XX(134))/(JVS(910))
  XX(138) = (X(138)-JVS(115)*XX(46)-JVS(321)*XX(90)-JVS(354)*XX(96)-JVS(371)*XX(99)-JVS(400)*XX(102)-JVS(406)*XX(103)&
              &-JVS(420)*XX(105)-JVS(476)*XX(109)-JVS(503)*XX(110)-JVS(518)*XX(112)-JVS(541)*XX(113)-JVS(558)*XX(114)&
              &-JVS(588)*XX(117)-JVS(601)*XX(118)-JVS(622)*XX(119)-JVS(715)*XX(127)-JVS(770)*XX(131)-JVS(848)*XX(134)&
              &-JVS(899)*XX(136)-JVS(911)*XX(137))/(JVS(922))
  XX(139) = (X(139)-JVS(288)*XX(83)-JVS(350)*XX(95)-JVS(504)*XX(110)-JVS(542)*XX(113)-JVS(559)*XX(114)-JVS(589)*XX(117)&
              &-JVS(623)*XX(119)-JVS(716)*XX(127)-JVS(771)*XX(131)-JVS(849)*XX(134))/(JVS(935))
  XX(140) = (X(140)-JVS(31)*XX(16)-JVS(116)*XX(46)-JVS(147)*XX(47)-JVS(477)*XX(109)-JVS(717)*XX(127)-JVS(850)*XX(134))&
              &/(JVS(967))
  XX(141) = (X(141)-JVS(313)*XX(88)-JVS(401)*XX(102)-JVS(428)*XX(106)-JVS(478)*XX(109)-JVS(567)*XX(115)-JVS(676)*XX(125)&
              &-JVS(740)*XX(129)-JVS(785)*XX(132)-JVS(851)*XX(134)-JVS(881)*XX(135)-JVS(968)*XX(140))/(JVS(989))
  XX(142) = (X(142)-JVS(344)*XX(94)-JVS(519)*XX(112)-JVS(590)*XX(117)-JVS(602)*XX(118)-JVS(624)*XX(119)-JVS(718)*XX(127)&
              &-JVS(796)*XX(133)-JVS(852)*XX(134)-JVS(882)*XX(135)-JVS(900)*XX(136)-JVS(912)*XX(137)-JVS(936)*XX(139)&
              &-JVS(969)*XX(140)-JVS(990)*XX(141))/(JVS(1004))
  XX(143) = (X(143)-JVS(29)*XX(15)-JVS(148)*XX(47)-JVS(479)*XX(109)-JVS(543)*XX(113)-JVS(642)*XX(121)-JVS(656)*XX(123)&
              &-JVS(719)*XX(127)-JVS(772)*XX(131)-JVS(853)*XX(134)-JVS(937)*XX(139)-JVS(970)*XX(140))/(JVS(1016))
  XX(144) = (X(144)-JVS(117)*XX(46)-JVS(270)*XX(79)-JVS(412)*XX(104)-JVS(480)*XX(109)-JVS(577)*XX(116)-JVS(854)*XX(134)&
              &-JVS(971)*XX(140))/(JVS(1030))
  XX(145) = (X(145)-JVS(402)*XX(102)-JVS(855)*XX(134)-JVS(883)*XX(135)-JVS(991)*XX(141))/(JVS(1046))
  XX(146) = (X(146)-JVS(481)*XX(109)-JVS(544)*XX(113)-JVS(741)*XX(129)-JVS(757)*XX(130)-JVS(856)*XX(134)-JVS(884)&
              &*XX(135))/(JVS(1085))
  XX(147) = (X(147)-JVS(15)*XX(8)-JVS(118)*XX(46)-JVS(149)*XX(47)-JVS(206)*XX(65)-JVS(289)*XX(83)-JVS(337)*XX(93)&
              &-JVS(355)*XX(96)-JVS(359)*XX(97)-JVS(365)*XX(98)-JVS(385)*XX(101)-JVS(413)*XX(104)-JVS(421)*XX(105)-JVS(444)&
              &*XX(108)-JVS(482)*XX(109)-JVS(505)*XX(110)-JVS(520)*XX(112)-JVS(545)*XX(113)-JVS(560)*XX(114)-JVS(591)&
              &*XX(117)-JVS(603)*XX(118)-JVS(625)*XX(119)-JVS(666)*XX(124)-JVS(688)*XX(126)-JVS(720)*XX(127)-JVS(773)&
              &*XX(131)-JVS(797)*XX(133)-JVS(857)*XX(134)-JVS(885)*XX(135)-JVS(901)*XX(136)-JVS(913)*XX(137)-JVS(923)&
              &*XX(138)-JVS(938)*XX(139)-JVS(972)*XX(140)-JVS(992)*XX(141)-JVS(1005)*XX(142)-JVS(1017)*XX(143)-JVS(1031)&
              &*XX(144)-JVS(1047)*XX(145)-JVS(1086)*XX(146))/(JVS(1115))
  XX(148) = (X(148)-JVS(119)*XX(46)-JVS(199)*XX(63)-JVS(214)*XX(67)-JVS(331)*XX(92)-JVS(338)*XX(93)-JVS(386)*XX(101)&
              &-JVS(407)*XX(103)-JVS(429)*XX(106)-JVS(483)*XX(109)-JVS(506)*XX(110)-JVS(546)*XX(113)-JVS(568)*XX(115)&
              &-JVS(578)*XX(116)-JVS(592)*XX(117)-JVS(604)*XX(118)-JVS(626)*XX(119)-JVS(635)*XX(120)-JVS(643)*XX(121)&
              &-JVS(657)*XX(123)-JVS(667)*XX(124)-JVS(677)*XX(125)-JVS(689)*XX(126)-JVS(721)*XX(127)-JVS(729)*XX(128)&
              &-JVS(742)*XX(129)-JVS(758)*XX(130)-JVS(774)*XX(131)-JVS(786)*XX(132)-JVS(798)*XX(133)-JVS(858)*XX(134)&
              &-JVS(886)*XX(135)-JVS(902)*XX(136)-JVS(914)*XX(137)-JVS(924)*XX(138)-JVS(939)*XX(139)-JVS(973)*XX(140)&
              &-JVS(993)*XX(141)-JVS(1006)*XX(142)-JVS(1018)*XX(143)-JVS(1032)*XX(144)-JVS(1048)*XX(145)-JVS(1087)*XX(146)&
              &-JVS(1116)*XX(147))/(JVS(1164))
  XX(149) = (X(149)-JVS(547)*XX(113)-JVS(730)*XX(128)-JVS(743)*XX(129)-JVS(759)*XX(130)-JVS(787)*XX(132)-JVS(859)&
              &*XX(134)-JVS(887)*XX(135)-JVS(974)*XX(140)-JVS(1088)*XX(146)-JVS(1117)*XX(147)-JVS(1165)*XX(148))/(JVS(1197))
  XX(150) = (X(150)-JVS(36)*XX(18)-JVS(39)*XX(19)-JVS(42)*XX(20)-JVS(72)*XX(30)-JVS(76)*XX(31)-JVS(87)*XX(36)-JVS(120)&
              &*XX(46)-JVS(153)*XX(48)-JVS(155)*XX(49)-JVS(157)*XX(50)-JVS(159)*XX(51)-JVS(161)*XX(52)-JVS(163)*XX(53)&
              &-JVS(172)*XX(56)-JVS(178)*XX(58)-JVS(182)*XX(59)-JVS(187)*XX(60)-JVS(192)*XX(61)-JVS(202)*XX(64)-JVS(207)&
              &*XX(65)-JVS(210)*XX(66)-JVS(215)*XX(67)-JVS(221)*XX(69)-JVS(224)*XX(70)-JVS(228)*XX(71)-JVS(232)*XX(72)&
              &-JVS(235)*XX(73)-JVS(240)*XX(74)-JVS(243)*XX(75)-JVS(250)*XX(76)-JVS(257)*XX(77)-JVS(266)*XX(78)-JVS(271)&
              &*XX(79)-JVS(276)*XX(80)-JVS(280)*XX(81)-JVS(290)*XX(83)-JVS(295)*XX(84)-JVS(298)*XX(85)-JVS(303)*XX(86)&
              &-JVS(308)*XX(87)-JVS(314)*XX(88)-JVS(318)*XX(89)-JVS(322)*XX(90)-JVS(326)*XX(91)-JVS(332)*XX(92)-JVS(339)&
              &*XX(93)-JVS(345)*XX(94)-JVS(351)*XX(95)-JVS(356)*XX(96)-JVS(360)*XX(97)-JVS(366)*XX(98)-JVS(372)*XX(99)&
              &-JVS(378)*XX(100)-JVS(387)*XX(101)-JVS(403)*XX(102)-JVS(414)*XX(104)-JVS(422)*XX(105)-JVS(430)*XX(106)&
              &-JVS(438)*XX(107)-JVS(445)*XX(108)-JVS(484)*XX(109)-JVS(507)*XX(110)-JVS(521)*XX(112)-JVS(548)*XX(113)&
              &-JVS(561)*XX(114)-JVS(569)*XX(115)-JVS(579)*XX(116)-JVS(593)*XX(117)-JVS(605)*XX(118)-JVS(627)*XX(119)&
              &-JVS(636)*XX(120)-JVS(644)*XX(121)-JVS(650)*XX(122)-JVS(668)*XX(124)-JVS(678)*XX(125)-JVS(690)*XX(126)&
              &-JVS(722)*XX(127)-JVS(731)*XX(128)-JVS(744)*XX(129)-JVS(760)*XX(130)-JVS(775)*XX(131)-JVS(788)*XX(132)&
              &-JVS(799)*XX(133)-JVS(860)*XX(134)-JVS(888)*XX(135)-JVS(903)*XX(136)-JVS(915)*XX(137)-JVS(925)*XX(138)&
              &-JVS(940)*XX(139)-JVS(975)*XX(140)-JVS(994)*XX(141)-JVS(1007)*XX(142)-JVS(1019)*XX(143)-JVS(1033)*XX(144)&
              &-JVS(1049)*XX(145)-JVS(1089)*XX(146)-JVS(1118)*XX(147)-JVS(1166)*XX(148)-JVS(1198)*XX(149))/(JVS(1299))
  XX(151) = (X(151)-JVS(121)*XX(46)-JVS(218)*XX(68)-JVS(244)*XX(75)-JVS(404)*XX(102)-JVS(431)*XX(106)-JVS(439)*XX(107)&
              &-JVS(485)*XX(109)-JVS(508)*XX(110)-JVS(513)*XX(111)-JVS(522)*XX(112)-JVS(570)*XX(115)-JVS(580)*XX(116)&
              &-JVS(594)*XX(117)-JVS(606)*XX(118)-JVS(628)*XX(119)-JVS(637)*XX(120)-JVS(645)*XX(121)-JVS(658)*XX(123)&
              &-JVS(669)*XX(124)-JVS(679)*XX(125)-JVS(691)*XX(126)-JVS(723)*XX(127)-JVS(732)*XX(128)-JVS(745)*XX(129)&
              &-JVS(761)*XX(130)-JVS(776)*XX(131)-JVS(789)*XX(132)-JVS(800)*XX(133)-JVS(861)*XX(134)-JVS(889)*XX(135)&
              &-JVS(904)*XX(136)-JVS(916)*XX(137)-JVS(926)*XX(138)-JVS(941)*XX(139)-JVS(976)*XX(140)-JVS(995)*XX(141)&
              &-JVS(1008)*XX(142)-JVS(1020)*XX(143)-JVS(1034)*XX(144)-JVS(1050)*XX(145)-JVS(1090)*XX(146)-JVS(1119)*XX(147)&
              &-JVS(1167)*XX(148)-JVS(1199)*XX(149)-JVS(1300)*XX(150))/(JVS(1346))
  XX(152) = (X(152)-JVS(48)*XX(22)-JVS(54)*XX(24)-JVS(60)*XX(26)-JVS(66)*XX(28)-JVS(73)*XX(30)-JVS(122)*XX(46)-JVS(183)&
              &*XX(59)-JVS(188)*XX(60)-JVS(193)*XX(61)-JVS(208)*XX(65)-JVS(216)*XX(67)-JVS(229)*XX(71)-JVS(233)*XX(72)&
              &-JVS(236)*XX(73)-JVS(241)*XX(74)-JVS(245)*XX(75)-JVS(251)*XX(76)-JVS(258)*XX(77)-JVS(267)*XX(78)-JVS(272)&
              &*XX(79)-JVS(277)*XX(80)-JVS(281)*XX(81)-JVS(291)*XX(83)-JVS(296)*XX(84)-JVS(304)*XX(86)-JVS(309)*XX(87)&
              &-JVS(315)*XX(88)-JVS(319)*XX(89)-JVS(323)*XX(90)-JVS(327)*XX(91)-JVS(333)*XX(92)-JVS(346)*XX(94)-JVS(352)&
              &*XX(95)-JVS(373)*XX(99)-JVS(379)*XX(100)-JVS(388)*XX(101)-JVS(408)*XX(103)-JVS(423)*XX(105)-JVS(432)*XX(106)&
              &-JVS(440)*XX(107)-JVS(446)*XX(108)-JVS(486)*XX(109)-JVS(509)*XX(110)-JVS(514)*XX(111)-JVS(549)*XX(113)&
              &-JVS(562)*XX(114)-JVS(571)*XX(115)-JVS(581)*XX(116)-JVS(595)*XX(117)-JVS(607)*XX(118)-JVS(629)*XX(119)&
              &-JVS(638)*XX(120)-JVS(646)*XX(121)-JVS(651)*XX(122)-JVS(659)*XX(123)-JVS(670)*XX(124)-JVS(680)*XX(125)&
              &-JVS(692)*XX(126)-JVS(724)*XX(127)-JVS(733)*XX(128)-JVS(746)*XX(129)-JVS(762)*XX(130)-JVS(777)*XX(131)&
              &-JVS(790)*XX(132)-JVS(862)*XX(134)-JVS(890)*XX(135)-JVS(905)*XX(136)-JVS(917)*XX(137)-JVS(927)*XX(138)&
              &-JVS(942)*XX(139)-JVS(977)*XX(140)-JVS(996)*XX(141)-JVS(1009)*XX(142)-JVS(1021)*XX(143)-JVS(1035)*XX(144)&
              &-JVS(1051)*XX(145)-JVS(1091)*XX(146)-JVS(1120)*XX(147)-JVS(1168)*XX(148)-JVS(1200)*XX(149)-JVS(1301)*XX(150)&
              &-JVS(1347)*XX(151))/(JVS(1429))
  XX(153) = (X(153)-JVS(168)*XX(55)-JVS(225)*XX(70)-JVS(259)*XX(77)-JVS(268)*XX(78)-JVS(284)*XX(82)-JVS(299)*XX(85)&
              &-JVS(433)*XX(106)-JVS(447)*XX(108)-JVS(487)*XX(109)-JVS(550)*XX(113)-JVS(572)*XX(115)-JVS(681)*XX(125)&
              &-JVS(747)*XX(129)-JVS(791)*XX(132)-JVS(863)*XX(134)-JVS(891)*XX(135)-JVS(978)*XX(140)-JVS(1092)*XX(146)&
              &-JVS(1121)*XX(147)-JVS(1169)*XX(148)-JVS(1201)*XX(149)-JVS(1302)*XX(150)-JVS(1348)*XX(151)-JVS(1430)*XX(152))&
              &/(JVS(1467))
  XX(154) = (X(154)-JVS(13)*XX(7)-JVS(150)*XX(47)-JVS(166)*XX(54)-JVS(169)*XX(55)-JVS(175)*XX(57)-JVS(196)*XX(62)&
              &-JVS(200)*XX(63)-JVS(211)*XX(66)-JVS(219)*XX(68)-JVS(237)*XX(73)-JVS(285)*XX(82)-JVS(380)*XX(100)-JVS(415)&
              &*XX(104)-JVS(448)*XX(108)-JVS(488)*XX(109)-JVS(551)*XX(113)-JVS(652)*XX(122)-JVS(725)*XX(127)-JVS(763)&
              &*XX(130)-JVS(864)*XX(134)-JVS(892)*XX(135)-JVS(979)*XX(140)-JVS(1036)*XX(144)-JVS(1093)*XX(146)-JVS(1122)&
              &*XX(147)-JVS(1170)*XX(148)-JVS(1202)*XX(149)-JVS(1303)*XX(150)-JVS(1349)*XX(151)-JVS(1431)*XX(152)-JVS(1468)&
              &*XX(153))/(JVS(1536))
  XX(155) = (X(155)-JVS(45)*XX(21)-JVS(179)*XX(58)-JVS(197)*XX(62)-JVS(203)*XX(64)-JVS(222)*XX(69)-JVS(300)*XX(85)&
              &-JVS(361)*XX(97)-JVS(367)*XX(98)-JVS(449)*XX(108)-JVS(489)*XX(109)-JVS(515)*XX(111)-JVS(552)*XX(113)-JVS(608)&
              &*XX(118)-JVS(630)*XX(119)-JVS(653)*XX(122)-JVS(660)*XX(123)-JVS(671)*XX(124)-JVS(693)*XX(126)-JVS(726)&
              &*XX(127)-JVS(734)*XX(128)-JVS(748)*XX(129)-JVS(764)*XX(130)-JVS(778)*XX(131)-JVS(792)*XX(132)-JVS(801)&
              &*XX(133)-JVS(865)*XX(134)-JVS(893)*XX(135)-JVS(906)*XX(136)-JVS(918)*XX(137)-JVS(928)*XX(138)-JVS(943)&
              &*XX(139)-JVS(980)*XX(140)-JVS(997)*XX(141)-JVS(1010)*XX(142)-JVS(1022)*XX(143)-JVS(1037)*XX(144)-JVS(1052)&
              &*XX(145)-JVS(1094)*XX(146)-JVS(1123)*XX(147)-JVS(1171)*XX(148)-JVS(1203)*XX(149)-JVS(1304)*XX(150)-JVS(1350)&
              &*XX(151)-JVS(1432)*XX(152)-JVS(1469)*XX(153)-JVS(1537)*XX(154))/(JVS(1588))
  XX(156) = (X(156)-JVS(51)*XX(23)-JVS(57)*XX(25)-JVS(63)*XX(27)-JVS(69)*XX(29)-JVS(74)*XX(30)-JVS(123)*XX(46)-JVS(184)&
              &*XX(59)-JVS(189)*XX(60)-JVS(194)*XX(61)-JVS(212)*XX(66)-JVS(292)*XX(83)-JVS(334)*XX(92)-JVS(340)*XX(93)&
              &-JVS(357)*XX(96)-JVS(374)*XX(99)-JVS(381)*XX(100)-JVS(389)*XX(101)-JVS(409)*XX(103)-JVS(424)*XX(105)-JVS(434)&
              &*XX(106)-JVS(441)*XX(107)-JVS(450)*XX(108)-JVS(490)*XX(109)-JVS(510)*XX(110)-JVS(516)*XX(111)-JVS(523)&
              &*XX(112)-JVS(553)*XX(113)-JVS(563)*XX(114)-JVS(573)*XX(115)-JVS(582)*XX(116)-JVS(596)*XX(117)-JVS(609)&
              &*XX(118)-JVS(631)*XX(119)-JVS(639)*XX(120)-JVS(647)*XX(121)-JVS(654)*XX(122)-JVS(661)*XX(123)-JVS(672)&
              &*XX(124)-JVS(682)*XX(125)-JVS(694)*XX(126)-JVS(727)*XX(127)-JVS(735)*XX(128)-JVS(749)*XX(129)-JVS(765)&
              &*XX(130)-JVS(779)*XX(131)-JVS(793)*XX(132)-JVS(802)*XX(133)-JVS(866)*XX(134)-JVS(894)*XX(135)-JVS(907)&
              &*XX(136)-JVS(919)*XX(137)-JVS(929)*XX(138)-JVS(944)*XX(139)-JVS(981)*XX(140)-JVS(998)*XX(141)-JVS(1011)&
              &*XX(142)-JVS(1023)*XX(143)-JVS(1038)*XX(144)-JVS(1053)*XX(145)-JVS(1095)*XX(146)-JVS(1124)*XX(147)-JVS(1172)&
              &*XX(148)-JVS(1204)*XX(149)-JVS(1305)*XX(150)-JVS(1351)*XX(151)-JVS(1433)*XX(152)-JVS(1470)*XX(153)-JVS(1538)&
              &*XX(154)-JVS(1589)*XX(155))/(JVS(1637))
  XX(157) = (X(157)-JVS(25)*XX(13)-JVS(151)*XX(47)-JVS(435)*XX(106)-JVS(491)*XX(109)-JVS(574)*XX(115)-JVS(683)*XX(125)&
              &-JVS(750)*XX(129)-JVS(794)*XX(132)-JVS(867)*XX(134)-JVS(895)*XX(135)-JVS(982)*XX(140)-JVS(1054)*XX(145)&
              &-JVS(1096)*XX(146)-JVS(1125)*XX(147)-JVS(1173)*XX(148)-JVS(1205)*XX(149)-JVS(1306)*XX(150)-JVS(1352)*XX(151)&
              &-JVS(1434)*XX(152)-JVS(1471)*XX(153)-JVS(1539)*XX(154)-JVS(1590)*XX(155)-JVS(1638)*XX(156))/(JVS(1661))
  XX(157) = XX(157)
  XX(156) = XX(156)-JVS(1660)*XX(157)
  XX(155) = XX(155)-JVS(1636)*XX(156)-JVS(1659)*XX(157)
  XX(154) = XX(154)-JVS(1587)*XX(155)-JVS(1635)*XX(156)-JVS(1658)*XX(157)
  XX(153) = XX(153)-JVS(1535)*XX(154)-JVS(1586)*XX(155)-JVS(1634)*XX(156)-JVS(1657)*XX(157)
  XX(152) = XX(152)-JVS(1466)*XX(153)-JVS(1534)*XX(154)-JVS(1585)*XX(155)-JVS(1633)*XX(156)-JVS(1656)*XX(157)
  XX(151) = XX(151)-JVS(1428)*XX(152)-JVS(1465)*XX(153)-JVS(1533)*XX(154)-JVS(1584)*XX(155)-JVS(1632)*XX(156)-JVS(1655)&
              &*XX(157)
  XX(150) = XX(150)-JVS(1345)*XX(151)-JVS(1427)*XX(152)-JVS(1464)*XX(153)-JVS(1532)*XX(154)-JVS(1583)*XX(155)-JVS(1631)&
              &*XX(156)-JVS(1654)*XX(157)
  XX(149) = XX(149)-JVS(1298)*XX(150)-JVS(1344)*XX(151)-JVS(1426)*XX(152)-JVS(1463)*XX(153)-JVS(1531)*XX(154)-JVS(1582)&
              &*XX(155)-JVS(1630)*XX(156)-JVS(1653)*XX(157)
  XX(148) = XX(148)-JVS(1196)*XX(149)-JVS(1297)*XX(150)-JVS(1343)*XX(151)-JVS(1425)*XX(152)-JVS(1462)*XX(153)-JVS(1530)&
              &*XX(154)-JVS(1581)*XX(155)-JVS(1629)*XX(156)-JVS(1652)*XX(157)
  XX(147) = XX(147)-JVS(1163)*XX(148)-JVS(1195)*XX(149)-JVS(1296)*XX(150)-JVS(1342)*XX(151)-JVS(1424)*XX(152)-JVS(1461)&
              &*XX(153)-JVS(1529)*XX(154)-JVS(1580)*XX(155)-JVS(1628)*XX(156)-JVS(1651)*XX(157)
  XX(146) = XX(146)-JVS(1114)*XX(147)-JVS(1162)*XX(148)-JVS(1295)*XX(150)-JVS(1341)*XX(151)-JVS(1423)*XX(152)-JVS(1460)&
              &*XX(153)-JVS(1528)*XX(154)-JVS(1579)*XX(155)-JVS(1627)*XX(156)
  XX(145) = XX(145)-JVS(1084)*XX(146)-JVS(1161)*XX(148)-JVS(1194)*XX(149)-JVS(1294)*XX(150)-JVS(1340)*XX(151)-JVS(1422)&
              &*XX(152)-JVS(1459)*XX(153)-JVS(1527)*XX(154)-JVS(1578)*XX(155)-JVS(1626)*XX(156)-JVS(1650)*XX(157)
  XX(144) = XX(144)-JVS(1083)*XX(146)-JVS(1113)*XX(147)-JVS(1160)*XX(148)-JVS(1193)*XX(149)-JVS(1293)*XX(150)-JVS(1339)&
              &*XX(151)-JVS(1421)*XX(152)-JVS(1458)*XX(153)-JVS(1526)*XX(154)-JVS(1577)*XX(155)-JVS(1625)*XX(156)
  XX(143) = XX(143)-JVS(1029)*XX(144)-JVS(1045)*XX(145)-JVS(1082)*XX(146)-JVS(1112)*XX(147)-JVS(1159)*XX(148)-JVS(1192)&
              &*XX(149)-JVS(1292)*XX(150)-JVS(1338)*XX(151)-JVS(1420)*XX(152)-JVS(1457)*XX(153)-JVS(1525)*XX(154)-JVS(1576)&
              &*XX(155)-JVS(1624)*XX(156)-JVS(1649)*XX(157)
  XX(142) = XX(142)-JVS(1015)*XX(143)-JVS(1028)*XX(144)-JVS(1044)*XX(145)-JVS(1081)*XX(146)-JVS(1111)*XX(147)-JVS(1158)&
              &*XX(148)-JVS(1191)*XX(149)-JVS(1291)*XX(150)-JVS(1337)*XX(151)-JVS(1419)*XX(152)-JVS(1456)*XX(153)-JVS(1524)&
              &*XX(154)-JVS(1575)*XX(155)-JVS(1623)*XX(156)-JVS(1648)*XX(157)
  XX(141) = XX(141)-JVS(1080)*XX(146)-JVS(1157)*XX(148)-JVS(1190)*XX(149)-JVS(1290)*XX(150)-JVS(1336)*XX(151)-JVS(1418)&
              &*XX(152)-JVS(1455)*XX(153)-JVS(1523)*XX(154)-JVS(1574)*XX(155)-JVS(1622)*XX(156)-JVS(1647)*XX(157)
  XX(140) = XX(140)-JVS(1156)*XX(148)-JVS(1289)*XX(150)-JVS(1335)*XX(151)-JVS(1417)*XX(152)-JVS(1454)*XX(153)-JVS(1522)&
              &*XX(154)-JVS(1573)*XX(155)
  XX(139) = XX(139)-JVS(966)*XX(140)-JVS(1043)*XX(145)-JVS(1079)*XX(146)-JVS(1155)*XX(148)-JVS(1189)*XX(149)-JVS(1288)&
              &*XX(150)-JVS(1334)*XX(151)-JVS(1416)*XX(152)-JVS(1453)*XX(153)-JVS(1521)*XX(154)-JVS(1572)*XX(155)-JVS(1621)&
              &*XX(156)
  XX(138) = XX(138)-JVS(934)*XX(139)-JVS(965)*XX(140)-JVS(988)*XX(141)-JVS(1014)*XX(143)-JVS(1042)*XX(145)-JVS(1078)&
              &*XX(146)-JVS(1110)*XX(147)-JVS(1154)*XX(148)-JVS(1188)*XX(149)-JVS(1287)*XX(150)-JVS(1333)*XX(151)-JVS(1415)&
              &*XX(152)-JVS(1452)*XX(153)-JVS(1520)*XX(154)-JVS(1571)*XX(155)-JVS(1620)*XX(156)
  XX(137) = XX(137)-JVS(964)*XX(140)-JVS(987)*XX(141)-JVS(1077)*XX(146)-JVS(1109)*XX(147)-JVS(1153)*XX(148)-JVS(1187)&
              &*XX(149)-JVS(1286)*XX(150)-JVS(1332)*XX(151)-JVS(1414)*XX(152)-JVS(1451)*XX(153)-JVS(1519)*XX(154)-JVS(1570)&
              &*XX(155)-JVS(1619)*XX(156)
  XX(136) = XX(136)-JVS(963)*XX(140)-JVS(1076)*XX(146)-JVS(1108)*XX(147)-JVS(1152)*XX(148)-JVS(1186)*XX(149)-JVS(1285)&
              &*XX(150)-JVS(1331)*XX(151)-JVS(1413)*XX(152)-JVS(1450)*XX(153)-JVS(1518)*XX(154)-JVS(1569)*XX(155)-JVS(1618)&
              &*XX(156)
  XX(135) = XX(135)-JVS(1151)*XX(148)-JVS(1284)*XX(150)-JVS(1330)*XX(151)-JVS(1412)*XX(152)-JVS(1449)*XX(153)-JVS(1517)&
              &*XX(154)-JVS(1568)*XX(155)
  XX(134) = XX(134)-JVS(1283)*XX(150)-JVS(1411)*XX(152)-JVS(1448)*XX(153)-JVS(1516)*XX(154)-JVS(1567)*XX(155)
  XX(133) = XX(133)-JVS(843)*XX(134)-JVS(878)*XX(135)-JVS(962)*XX(140)-JVS(1003)*XX(142)-JVS(1027)*XX(144)-JVS(1075)&
              &*XX(146)-JVS(1107)*XX(147)-JVS(1150)*XX(148)-JVS(1185)*XX(149)-JVS(1282)*XX(150)-JVS(1329)*XX(151)-JVS(1410)&
              &*XX(152)-JVS(1515)*XX(154)-JVS(1566)*XX(155)-JVS(1617)*XX(156)-JVS(1646)*XX(157)
  XX(132) = XX(132)-JVS(842)*XX(134)-JVS(961)*XX(140)-JVS(1149)*XX(148)-JVS(1184)*XX(149)-JVS(1281)*XX(150)-JVS(1328)&
              &*XX(151)-JVS(1409)*XX(152)-JVS(1514)*XX(154)-JVS(1565)*XX(155)-JVS(1616)*XX(156)-JVS(1645)*XX(157)
  XX(131) = XX(131)-JVS(841)*XX(134)-JVS(933)*XX(139)-JVS(960)*XX(140)-JVS(1041)*XX(145)-JVS(1074)*XX(146)-JVS(1148)&
              &*XX(148)-JVS(1280)*XX(150)-JVS(1408)*XX(152)-JVS(1513)*XX(154)-JVS(1564)*XX(155)-JVS(1615)*XX(156)
  XX(130) = XX(130)-JVS(840)*XX(134)-JVS(877)*XX(135)-JVS(1106)*XX(147)-JVS(1147)*XX(148)-JVS(1279)*XX(150)-JVS(1327)&
              &*XX(151)-JVS(1407)*XX(152)-JVS(1512)*XX(154)-JVS(1614)*XX(156)
  XX(129) = XX(129)-JVS(839)*XX(134)-JVS(876)*XX(135)-JVS(1146)*XX(148)-JVS(1278)*XX(150)-JVS(1326)*XX(151)-JVS(1406)&
              &*XX(152)-JVS(1511)*XX(154)-JVS(1613)*XX(156)
  XX(128) = XX(128)-JVS(783)*XX(132)-JVS(838)*XX(134)-JVS(875)*XX(135)-JVS(1073)*XX(146)-JVS(1145)*XX(148)-JVS(1183)&
              &*XX(149)-JVS(1277)*XX(150)-JVS(1325)*XX(151)-JVS(1405)*XX(152)-JVS(1510)*XX(154)-JVS(1563)*XX(155)-JVS(1612)&
              &*XX(156)-JVS(1644)*XX(157)
  XX(127) = XX(127)-JVS(1144)*XX(148)-JVS(1276)*XX(150)-JVS(1404)*XX(152)-JVS(1509)*XX(154)-JVS(1562)*XX(155)
  XX(126) = XX(126)-JVS(837)*XX(134)-JVS(874)*XX(135)-JVS(959)*XX(140)-JVS(1072)*XX(146)-JVS(1143)*XX(148)-JVS(1275)&
              &*XX(150)-JVS(1324)*XX(151)-JVS(1403)*XX(152)-JVS(1508)*XX(154)-JVS(1611)*XX(156)
  XX(125) = XX(125)-JVS(782)*XX(132)-JVS(836)*XX(134)-JVS(1071)*XX(146)-JVS(1142)*XX(148)-JVS(1274)*XX(150)-JVS(1323)&
              &*XX(151)-JVS(1402)*XX(152)-JVS(1447)*XX(153)-JVS(1507)*XX(154)-JVS(1610)*XX(156)
  XX(124) = XX(124)-JVS(708)*XX(127)-JVS(835)*XX(134)-JVS(873)*XX(135)-JVS(1070)*XX(146)-JVS(1141)*XX(148)-JVS(1273)&
              &*XX(150)-JVS(1322)*XX(151)-JVS(1401)*XX(152)-JVS(1506)*XX(154)-JVS(1609)*XX(156)
  XX(123) = XX(123)-JVS(707)*XX(127)-JVS(768)*XX(131)-JVS(834)*XX(134)-JVS(932)*XX(139)-JVS(1040)*XX(145)-JVS(1069)&
              &*XX(146)-JVS(1140)*XX(148)-JVS(1272)*XX(150)-JVS(1321)*XX(151)-JVS(1400)*XX(152)-JVS(1505)*XX(154)-JVS(1608)&
              &*XX(156)-JVS(1643)*XX(157)
  XX(122) = XX(122)-JVS(753)*XX(130)-JVS(833)*XX(134)-JVS(872)*XX(135)-JVS(958)*XX(140)-JVS(1026)*XX(144)-JVS(1105)&
              &*XX(147)-JVS(1139)*XX(148)-JVS(1271)*XX(150)-JVS(1399)*XX(152)-JVS(1504)*XX(154)-JVS(1561)*XX(155)-JVS(1607)&
              &*XX(156)-JVS(1642)*XX(157)
  XX(121) = XX(121)-JVS(706)*XX(127)-JVS(832)*XX(134)-JVS(957)*XX(140)-JVS(1068)*XX(146)-JVS(1138)*XX(148)-JVS(1182)&
              &*XX(149)-JVS(1270)*XX(150)-JVS(1320)*XX(151)-JVS(1398)*XX(152)-JVS(1503)*XX(154)-JVS(1560)*XX(155)-JVS(1606)&
              &*XX(156)
  XX(120) = XX(120)-JVS(705)*XX(127)-JVS(831)*XX(134)-JVS(1067)*XX(146)-JVS(1137)*XX(148)-JVS(1181)*XX(149)-JVS(1269)&
              &*XX(150)-JVS(1319)*XX(151)-JVS(1397)*XX(152)-JVS(1502)*XX(154)-JVS(1559)*XX(155)-JVS(1605)*XX(156)
  XX(119) = XX(119)-JVS(830)*XX(134)-JVS(1268)*XX(150)-JVS(1396)*XX(152)-JVS(1501)*XX(154)-JVS(1558)*XX(155)
  XX(118) = XX(118)-JVS(616)*XX(119)-JVS(704)*XX(127)-JVS(829)*XX(134)-JVS(909)*XX(137)-JVS(956)*XX(140)-JVS(1104)&
              &*XX(147)-JVS(1267)*XX(150)-JVS(1395)*XX(152)
  XX(117) = XX(117)-JVS(703)*XX(127)-JVS(828)*XX(134)-JVS(1266)*XX(150)-JVS(1318)*XX(151)-JVS(1394)*XX(152)-JVS(1500)&
              &*XX(154)
  XX(116) = XX(116)-JVS(827)*XX(134)-JVS(955)*XX(140)-JVS(1066)*XX(146)-JVS(1136)*XX(148)-JVS(1180)*XX(149)-JVS(1265)&
              &*XX(150)-JVS(1317)*XX(151)-JVS(1393)*XX(152)-JVS(1499)*XX(154)-JVS(1557)*XX(155)-JVS(1604)*XX(156)
  XX(115) = XX(115)-JVS(826)*XX(134)-JVS(1065)*XX(146)-JVS(1135)*XX(148)-JVS(1264)*XX(150)-JVS(1316)*XX(151)-JVS(1392)&
              &*XX(152)-JVS(1498)*XX(154)-JVS(1603)*XX(156)
  XX(114) = XX(114)-JVS(702)*XX(127)-JVS(825)*XX(134)-JVS(954)*XX(140)-JVS(1263)*XX(150)-JVS(1391)*XX(152)-JVS(1497)&
              &*XX(154)-JVS(1556)*XX(155)-JVS(1602)*XX(156)
  XX(113) = XX(113)-JVS(1262)*XX(150)-JVS(1496)*XX(154)-JVS(1555)*XX(155)
  XX(112) = XX(112)-JVS(584)*XX(117)-JVS(599)*XX(118)-JVS(615)*XX(119)-JVS(824)*XX(134)-JVS(931)*XX(139)-JVS(953)&
              &*XX(140)-JVS(1103)*XX(147)-JVS(1261)*XX(150)-JVS(1390)*XX(152)-JVS(1495)*XX(154)
  XX(111) = XX(111)-JVS(614)*XX(119)-JVS(823)*XX(134)-JVS(1002)*XX(142)-JVS(1064)*XX(146)-JVS(1179)*XX(149)-JVS(1260)&
              &*XX(150)-JVS(1315)*XX(151)-JVS(1389)*XX(152)-JVS(1494)*XX(154)-JVS(1554)*XX(155)-JVS(1601)*XX(156)-JVS(1641)&
              &*XX(157)
  XX(110) = XX(110)-JVS(613)*XX(119)-JVS(822)*XX(134)-JVS(1259)*XX(150)-JVS(1388)*XX(152)
  XX(109) = XX(109)-JVS(1258)*XX(150)-JVS(1387)*XX(152)
  XX(108) = XX(108)-JVS(531)*XX(113)-JVS(1102)*XX(147)-JVS(1257)*XX(150)-JVS(1386)*XX(152)-JVS(1446)*XX(153)-JVS(1493)&
              &*XX(154)-JVS(1553)*XX(155)-JVS(1600)*XX(156)
  XX(107) = XX(107)-JVS(612)*XX(119)-JVS(821)*XX(134)-JVS(1001)*XX(142)-JVS(1063)*XX(146)-JVS(1178)*XX(149)-JVS(1256)&
              &*XX(150)-JVS(1314)*XX(151)-JVS(1385)*XX(152)-JVS(1492)*XX(154)-JVS(1599)*XX(156)-JVS(1640)*XX(157)
  XX(106) = XX(106)-JVS(460)*XX(109)-JVS(781)*XX(132)-JVS(1134)*XX(148)-JVS(1255)*XX(150)-JVS(1313)*XX(151)-JVS(1445)&
              &*XX(153)
  XX(105) = XX(105)-JVS(496)*XX(110)-JVS(701)*XX(127)-JVS(820)*XX(134)-JVS(1133)*XX(148)-JVS(1254)*XX(150)-JVS(1491)&
              &*XX(154)-JVS(1552)*XX(155)
  XX(104) = XX(104)-JVS(459)*XX(109)-JVS(819)*XX(134)-JVS(952)*XX(140)-JVS(1025)*XX(144)-JVS(1101)*XX(147)-JVS(1132)&
              &*XX(148)-JVS(1253)*XX(150)-JVS(1490)*XX(154)-JVS(1551)*XX(155)
  XX(103) = XX(103)-JVS(495)*XX(110)-JVS(530)*XX(113)-JVS(611)*XX(119)-JVS(700)*XX(127)-JVS(951)*XX(140)-JVS(986)&
              &*XX(141)-JVS(1062)*XX(146)-JVS(1252)*XX(150)-JVS(1384)*XX(152)-JVS(1489)*XX(154)-JVS(1598)*XX(156)
  XX(102) = XX(102)-JVS(1061)*XX(146)-JVS(1251)*XX(150)-JVS(1383)*XX(152)
  XX(101) = XX(101)-JVS(529)*XX(113)-JVS(985)*XX(141)-JVS(1060)*XX(146)-JVS(1250)*XX(150)-JVS(1382)*XX(152)-JVS(1488)&
              &*XX(154)-JVS(1597)*XX(156)
  XX(100) = XX(100)-JVS(699)*XX(127)-JVS(818)*XX(134)-JVS(950)*XX(140)-JVS(1249)*XX(150)-JVS(1381)*XX(152)-JVS(1487)&
              &*XX(154)-JVS(1550)*XX(155)-JVS(1596)*XX(156)
  XX(99) = XX(99)-JVS(458)*XX(109)-JVS(494)*XX(110)-JVS(610)*XX(119)-JVS(698)*XX(127)-JVS(817)*XX(134)-JVS(949)*XX(140)&
             &-JVS(1248)*XX(150)-JVS(1380)*XX(152)-JVS(1486)*XX(154)-JVS(1595)*XX(156)
  XX(98) = XX(98)-JVS(457)*XX(109)-JVS(663)*XX(124)-JVS(685)*XX(126)-JVS(816)*XX(134)-JVS(871)*XX(135)-JVS(1100)*XX(147)&
             &-JVS(1247)*XX(150)-JVS(1379)*XX(152)-JVS(1549)*XX(155)
  XX(97) = XX(97)-JVS(362)*XX(98)-JVS(456)*XX(109)-JVS(815)*XX(134)-JVS(897)*XX(136)-JVS(921)*XX(138)-JVS(1000)*XX(142)&
             &-JVS(1013)*XX(143)-JVS(1099)*XX(147)-JVS(1131)*XX(148)-JVS(1246)*XX(150)-JVS(1312)*XX(151)-JVS(1378)*XX(152)&
             &-JVS(1548)*XX(155)
  XX(96) = XX(96)-JVS(416)*XX(105)-JVS(554)*XX(114)-JVS(598)*XX(118)-JVS(767)*XX(131)-JVS(814)*XX(134)-JVS(1098)*XX(147)&
             &-JVS(1245)*XX(150)-JVS(1377)*XX(152)-JVS(1485)*XX(154)
  XX(95) = XX(95)-JVS(766)*XX(131)-JVS(930)*XX(139)-JVS(1039)*XX(145)-JVS(1059)*XX(146)-JVS(1244)*XX(150)-JVS(1376)&
             &*XX(152)-JVS(1484)*XX(154)
  XX(94) = XX(94)-JVS(999)*XX(142)-JVS(1058)*XX(146)-JVS(1177)*XX(149)-JVS(1243)*XX(150)-JVS(1375)*XX(152)-JVS(1483)&
             &*XX(154)-JVS(1639)*XX(157)
  XX(93) = XX(93)-JVS(382)*XX(101)-JVS(1097)*XX(147)-JVS(1176)*XX(149)-JVS(1242)*XX(150)-JVS(1374)*XX(152)
  XX(92) = XX(92)-JVS(455)*XX(109)-JVS(948)*XX(140)-JVS(1175)*XX(149)-JVS(1241)*XX(150)-JVS(1373)*XX(152)
  XX(91) = XX(91)-JVS(328)*XX(92)-JVS(454)*XX(109)-JVS(493)*XX(110)-JVS(697)*XX(127)-JVS(908)*XX(137)-JVS(947)*XX(140)&
             &-JVS(1240)*XX(150)-JVS(1372)*XX(152)
  XX(90) = XX(90)-JVS(369)*XX(99)-JVS(597)*XX(118)-JVS(813)*XX(134)-JVS(896)*XX(136)-JVS(920)*XX(138)-JVS(1012)*XX(143)&
             &-JVS(1239)*XX(150)-JVS(1371)*XX(152)
  XX(89) = XX(89)-JVS(492)*XX(110)-JVS(632)*XX(120)-JVS(696)*XX(127)-JVS(812)*XX(134)-JVS(1130)*XX(148)-JVS(1174)&
             &*XX(149)-JVS(1238)*XX(150)-JVS(1370)*XX(152)
  XX(88) = XX(88)-JVS(984)*XX(141)-JVS(1057)*XX(146)-JVS(1237)*XX(150)-JVS(1369)*XX(152)
  XX(87) = XX(87)-JVS(695)*XX(127)-JVS(780)*XX(132)-JVS(811)*XX(134)-JVS(1129)*XX(148)-JVS(1236)*XX(150)
  XX(86) = XX(86)-JVS(453)*XX(109)-JVS(640)*XX(121)-JVS(810)*XX(134)-JVS(946)*XX(140)-JVS(1235)*XX(150)-JVS(1368)&
             &*XX(152)
  XX(85) = XX(85)-JVS(452)*XX(109)-JVS(528)*XX(113)-JVS(737)*XX(129)-JVS(1234)*XX(150)-JVS(1444)*XX(153)-JVS(1547)&
             &*XX(155)
  XX(84) = XX(84)-JVS(684)*XX(126)-JVS(809)*XX(134)-JVS(870)*XX(135)-JVS(945)*XX(140)-JVS(1233)*XX(150)-JVS(1367)&
             &*XX(152)
  XX(83) = XX(83)-JVS(808)*XX(134)-JVS(1232)*XX(150)-JVS(1482)*XX(154)
  XX(82) = XX(82)-JVS(442)*XX(108)-JVS(527)*XX(113)-JVS(1231)*XX(150)-JVS(1443)*XX(153)-JVS(1481)*XX(154)-JVS(1546)&
             &*XX(155)
  XX(81) = XX(81)-JVS(583)*XX(117)-JVS(662)*XX(124)-JVS(1056)*XX(146)-JVS(1230)*XX(150)-JVS(1366)*XX(152)-JVS(1480)&
             &*XX(154)
  XX(80) = XX(80)-JVS(752)*XX(130)-JVS(869)*XX(135)-JVS(1229)*XX(150)-JVS(1365)*XX(152)
  XX(79) = XX(79)-JVS(575)*XX(116)-JVS(807)*XX(134)-JVS(1024)*XX(144)-JVS(1128)*XX(148)-JVS(1228)*XX(150)
  XX(78) = XX(78)-JVS(1227)*XX(150)-JVS(1442)*XX(153)
  XX(77) = XX(77)-JVS(1226)*XX(150)-JVS(1441)*XX(153)
  XX(76) = XX(76)-JVS(252)*XX(77)-JVS(1225)*XX(150)-JVS(1440)*XX(153)
  XX(75) = XX(75)-JVS(806)*XX(134)-JVS(1224)*XX(150)-JVS(1311)*XX(151)-JVS(1364)*XX(152)
  XX(74) = XX(74)-JVS(425)*XX(106)-JVS(674)*XX(125)-JVS(1223)*XX(150)-JVS(1363)*XX(152)
  XX(73) = XX(73)-JVS(1222)*XX(150)-JVS(1362)*XX(152)-JVS(1479)*XX(154)-JVS(1545)*XX(155)
  XX(72) = XX(72)-JVS(565)*XX(115)-JVS(1055)*XX(146)-JVS(1221)*XX(150)-JVS(1361)*XX(152)
  XX(71) = XX(71)-JVS(736)*XX(129)-JVS(868)*XX(135)-JVS(1220)*XX(150)-JVS(1360)*XX(152)
  XX(70) = XX(70)-JVS(260)*XX(78)-JVS(451)*XX(109)-JVS(564)*XX(115)-JVS(673)*XX(125)-JVS(1219)*XX(150)-JVS(1439)*XX(153)
  XX(69) = XX(69)-JVS(526)*XX(113)-JVS(805)*XX(134)-JVS(1218)*XX(150)-JVS(1310)*XX(151)-JVS(1359)*XX(152)-JVS(1544)&
             &*XX(155)
  XX(68) = XX(68)-JVS(804)*XX(134)-JVS(1309)*XX(151)-JVS(1358)*XX(152)-JVS(1478)*XX(154)-JVS(1543)*XX(155)
  XX(67) = XX(67)-JVS(1127)*XX(148)-JVS(1217)*XX(150)-JVS(1308)*XX(151)
  XX(66) = XX(66)-JVS(1216)*XX(150)-JVS(1477)*XX(154)-JVS(1594)*XX(156)
  XX(65) = XX(65)-JVS(1215)*XX(150)-JVS(1357)*XX(152)
  XX(64) = XX(64)-JVS(525)*XX(113)-JVS(983)*XX(141)-JVS(1214)*XX(150)-JVS(1542)*XX(155)
  XX(63) = XX(63)-JVS(1126)*XX(148)-JVS(1307)*XX(151)-JVS(1476)*XX(154)-JVS(1541)*XX(155)
  XX(62) = XX(62)-JVS(524)*XX(113)-JVS(1475)*XX(154)-JVS(1540)*XX(155)
  XX(61) = XX(61)-JVS(1356)*XX(152)-JVS(1593)*XX(156)
  XX(60) = XX(60)-JVS(1355)*XX(152)-JVS(1592)*XX(156)
  XX(59) = XX(59)-JVS(1354)*XX(152)-JVS(1591)*XX(156)
  XX(58) = XX(58)-JVS(1213)*XX(150)-JVS(1353)*XX(152)
  XX(57) = XX(57)-JVS(375)*XX(100)-JVS(1474)*XX(154)
  XX(56) = XX(56)-JVS(368)*XX(99)-JVS(1212)*XX(150)
  XX(55) = XX(55)-JVS(1438)*XX(153)-JVS(1473)*XX(154)
  XX(54) = XX(54)-JVS(751)*XX(130)-JVS(1472)*XX(154)
  XX(53) = XX(53)-JVS(180)*XX(59)-JVS(1211)*XX(150)
  XX(52) = XX(52)-JVS(1210)*XX(150)-JVS(1437)*XX(153)
  XX(51) = XX(51)-JVS(1209)*XX(150)-JVS(1436)*XX(153)
  XX(50) = XX(50)-JVS(1208)*XX(150)-JVS(1435)*XX(153)
  XX(49) = XX(49)-JVS(190)*XX(61)-JVS(1207)*XX(150)
  XX(48) = XX(48)-JVS(185)*XX(60)-JVS(1206)*XX(150)
  XX(47) = XX(47)
  XX(46) = XX(46)
  XX(45) = XX(45)
  XX(44) = XX(44)
  XX(43) = XX(43)
  XX(42) = XX(42)
  XX(41) = XX(41)
  XX(40) = XX(40)
  XX(39) = XX(39)
  XX(38) = XX(38)
  XX(37) = XX(37)
  XX(36) = XX(36)
  XX(35) = XX(35)
  XX(34) = XX(34)
  XX(33) = XX(33)
  XX(32) = XX(32)
  XX(31) = XX(31)
  XX(30) = XX(30)
  XX(29) = XX(29)
  XX(28) = XX(28)
  XX(27) = XX(27)
  XX(26) = XX(26)
  XX(25) = XX(25)
  XX(24) = XX(24)
  XX(23) = XX(23)
  XX(22) = XX(22)
  XX(21) = XX(21)
  XX(20) = XX(20)
  XX(19) = XX(19)
  XX(18) = XX(18)
  XX(17) = XX(17)
  XX(16) = XX(16)
  XX(15) = XX(15)
  XX(14) = XX(14)
  XX(13) = XX(13)
  XX(12) = XX(12)
  XX(11) = XX(11)
  XX(10) = XX(10)
  XX(9) = XX(9)
  XX(8) = XX(8)
  XX(7) = XX(7)
  XX(6) = XX(6)
  XX(5) = XX(5)
  XX(4) = XX(4)
  XX(3) = XX(3)
  XX(2) = XX(2)
  XX(1) = XX(1)-JVS(803)*XX(134)
      
END SUBROUTINE KppSolveTR

! End of KppSolveTR function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! BLAS_UTIL - BLAS-LIKE utility functions
!   Arguments :
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

!--------------------------------------------------------------
!
! BLAS/LAPACK-like subroutines used by the integration algorithms
! It is recommended to replace them by calls to the optimized
!      BLAS/LAPACK library for your machine
!
!  (C) Adrian Sandu, Aug. 2004
!      Virginia Polytechnic Institute and State University
!--------------------------------------------------------------


!--------------------------------------------------------------
      SUBROUTINE WCOPY(N,X,incX,Y,incY)
!--------------------------------------------------------------
!     copies a vector, x, to a vector, y:  y <- x
!     only for incX=incY=1
!     after BLAS
!     replace this by the function from the optimized BLAS implementation:
!         CALL  SCOPY(N,X,1,Y,1)   or   CALL  DCOPY(N,X,1,Y,1)
!--------------------------------------------------------------
!     USE gckpp_Precision
      
      INTEGER  :: i,incX,incY,M,MP1,N
      REAL(kind=dp) :: X(N),Y(N)

      IF (N.LE.0) RETURN

      M = MOD(N,8)
      IF( M .NE. 0 ) THEN
        DO i = 1,M
          Y(i) = X(i)
        END DO
        IF( N .LT. 8 ) RETURN
      END IF    
      MP1 = M+1
      DO i = MP1,N,8
        Y(i) = X(i)
        Y(i + 1) = X(i + 1)
        Y(i + 2) = X(i + 2)
        Y(i + 3) = X(i + 3)
        Y(i + 4) = X(i + 4)
        Y(i + 5) = X(i + 5)
        Y(i + 6) = X(i + 6)
        Y(i + 7) = X(i + 7)
      END DO

      END SUBROUTINE WCOPY


!--------------------------------------------------------------
      SUBROUTINE WAXPY(N,Alpha,X,incX,Y,incY)
!--------------------------------------------------------------
!     constant times a vector plus a vector: y <- y + Alpha*x
!     only for incX=incY=1
!     after BLAS
!     replace this by the function from the optimized BLAS implementation:
!         CALL SAXPY(N,Alpha,X,1,Y,1) or  CALL DAXPY(N,Alpha,X,1,Y,1)
!--------------------------------------------------------------

      INTEGER  :: i,incX,incY,M,MP1,N
      REAL(kind=dp) :: X(N),Y(N),Alpha
      REAL(kind=dp), PARAMETER :: ZERO = 0.0_dp

      IF (Alpha .EQ. ZERO) RETURN
      IF (N .LE. 0) RETURN

      M = MOD(N,4)
      IF( M .NE. 0 ) THEN
        DO i = 1,M
          Y(i) = Y(i) + Alpha*X(i)
        END DO
        IF( N .LT. 4 ) RETURN
      END IF
      MP1 = M + 1
      DO i = MP1,N,4
        Y(i) = Y(i) + Alpha*X(i)
        Y(i + 1) = Y(i + 1) + Alpha*X(i + 1)
        Y(i + 2) = Y(i + 2) + Alpha*X(i + 2)
        Y(i + 3) = Y(i + 3) + Alpha*X(i + 3)
      END DO
      
      END SUBROUTINE WAXPY



!--------------------------------------------------------------
      SUBROUTINE WSCAL(N,Alpha,X,incX)
!--------------------------------------------------------------
!     constant times a vector: x(1:N) <- Alpha*x(1:N) 
!     only for incX=incY=1
!     after BLAS
!     replace this by the function from the optimized BLAS implementation:
!         CALL SSCAL(N,Alpha,X,1) or  CALL DSCAL(N,Alpha,X,1)
!--------------------------------------------------------------

      INTEGER  :: i,incX,M,MP1,N
      REAL(kind=dp)  :: X(N),Alpha
      REAL(kind=dp), PARAMETER  :: ZERO=0.0_dp, ONE=1.0_dp

      IF (Alpha .EQ. ONE) RETURN
      IF (N .LE. 0) RETURN

      M = MOD(N,5)
      IF( M .NE. 0 ) THEN
        IF (Alpha .EQ. (-ONE)) THEN
          DO i = 1,M
            X(i) = -X(i)
          END DO
        ELSEIF (Alpha .EQ. ZERO) THEN
          DO i = 1,M
            X(i) = ZERO
          END DO
        ELSE
          DO i = 1,M
            X(i) = Alpha*X(i)
          END DO
        END IF
        IF( N .LT. 5 ) RETURN
      END IF
      MP1 = M + 1
      IF (Alpha .EQ. (-ONE)) THEN
        DO i = MP1,N,5
          X(i)     = -X(i)
          X(i + 1) = -X(i + 1)
          X(i + 2) = -X(i + 2)
          X(i + 3) = -X(i + 3)
          X(i + 4) = -X(i + 4)
        END DO
      ELSEIF (Alpha .EQ. ZERO) THEN
        DO i = MP1,N,5
          X(i)     = ZERO
          X(i + 1) = ZERO
          X(i + 2) = ZERO
          X(i + 3) = ZERO
          X(i + 4) = ZERO
        END DO
      ELSE
        DO i = MP1,N,5
          X(i)     = Alpha*X(i)
          X(i + 1) = Alpha*X(i + 1)
          X(i + 2) = Alpha*X(i + 2)
          X(i + 3) = Alpha*X(i + 3)
          X(i + 4) = Alpha*X(i + 4)
        END DO
      END IF

      END SUBROUTINE WSCAL

!--------------------------------------------------------------
      REAL(kind=dp) FUNCTION WLAMCH( C )
!--------------------------------------------------------------
!     returns epsilon machine
!     after LAPACK
!     replace this by the function from the optimized LAPACK implementation:
!          CALL SLAMCH('E') or CALL DLAMCH('E')
!--------------------------------------------------------------
!      USE gckpp_Precision

      CHARACTER ::  C
      INTEGER    :: i
      REAL(kind=dp), SAVE  ::  Eps
      REAL(kind=dp)  ::  Suma
      REAL(kind=dp), PARAMETER  ::  ONE=1.0_dp, HALF=0.5_dp
      LOGICAL, SAVE   ::  First=.TRUE.
      
!$OMP THREADPRIVATE( Eps, First )

      IF (First) THEN
        First = .FALSE.
        Eps = HALF**(16)
        DO i = 17, 80
          Eps = Eps*HALF
          CALL WLAMCH_ADD(ONE,Eps,Suma)
          IF (Suma.LE.ONE) GOTO 10
        END DO
        PRINT*,'ERROR IN WLAMCH. EPS < ',Eps
        RETURN
10      Eps = Eps*2
        i = i-1      
      END IF

      WLAMCH = Eps

      END FUNCTION WLAMCH
     
      SUBROUTINE WLAMCH_ADD( A, B, Suma )
!      USE gckpp_Precision
      
      REAL(kind=dp) A, B, Suma
      Suma = A + B

      END SUBROUTINE WLAMCH_ADD
!--------------------------------------------------------------


!--------------------------------------------------------------
      SUBROUTINE SET2ZERO(N,Y)
!--------------------------------------------------------------
!     copies zeros into the vector y:  y <- 0
!     after BLAS
!--------------------------------------------------------------
      
      INTEGER ::  i,M,MP1,N
      REAL(kind=dp) ::  Y(N)
      REAL(kind=dp), PARAMETER :: ZERO = 0.0d0

      IF (N.LE.0) RETURN

      M = MOD(N,8)
      IF( M .NE. 0 ) THEN
        DO i = 1,M
          Y(i) = ZERO
        END DO
        IF( N .LT. 8 ) RETURN
      END IF    
      MP1 = M+1
      DO i = MP1,N,8
        Y(i)     = ZERO
        Y(i + 1) = ZERO
        Y(i + 2) = ZERO
        Y(i + 3) = ZERO
        Y(i + 4) = ZERO
        Y(i + 5) = ZERO
        Y(i + 6) = ZERO
        Y(i + 7) = ZERO
      END DO

      END SUBROUTINE SET2ZERO


!--------------------------------------------------------------
      REAL(kind=dp) FUNCTION WDOT (N, DX, incX, DY, incY) 
!--------------------------------------------------------------
!     dot produce: wdot = x(1:N)*y(1:N) 
!     only for incX=incY=1
!     after BLAS
!     replace this by the function from the optimized BLAS implementation:
!         CALL SDOT(N,X,1,Y,1) or  CALL DDOT(N,X,1,Y,1)
!--------------------------------------------------------------
!      USE messy_mecca_kpp_Precision
!--------------------------------------------------------------
      IMPLICIT NONE
      INTEGER :: N, incX, incY
      REAL(kind=dp) :: DX(N), DY(N) 

      INTEGER :: i, IX, IY, M, MP1, NS
                                 
      WDOT = 0.0D0 
      IF (N .LE. 0) RETURN 
      IF (incX .EQ. incY) IF (incX-1) 5,20,60 
!                                                                       
!     Code for unequal or nonpositive increments.                       
!                                                                       
    5 IX = 1 
      IY = 1 
      IF (incX .LT. 0) IX = (-N+1)*incX + 1 
      IF (incY .LT. 0) IY = (-N+1)*incY + 1 
      DO i = 1,N 
        WDOT = WDOT + DX(IX)*DY(IY) 
        IX = IX + incX 
        IY = IY + incY 
      END DO 
      RETURN 
!                                                                       
!     Code for both increments equal to 1.                              
!                                                                       
!     Clean-up loop so remaining vector length is a multiple of 5.      
!                                                                       
   20 M = MOD(N,5) 
      IF (M .EQ. 0) GO TO 40 
      DO i = 1,M 
         WDOT = WDOT + DX(i)*DY(i) 
      END DO 
      IF (N .LT. 5) RETURN 
   40 MP1 = M + 1 
      DO i = MP1,N,5 
          WDOT = WDOT + DX(i)*DY(i) + DX(i+1)*DY(i+1) + DX(i+2)*DY(i+2) +  &
                   DX(i+3)*DY(i+3) + DX(i+4)*DY(i+4)                   
      END DO 
      RETURN 
!                                                                       
!     Code for equal, positive, non-unit increments.                    
!                                                                       
   60 NS = N*incX 
      DO i = 1,NS,incX 
        WDOT = WDOT + DX(i)*DY(i) 
      END DO 

      END FUNCTION WDOT                                          


!--------------------------------------------------------------
      SUBROUTINE WADD(N,X,Y,Z)
!--------------------------------------------------------------
!     adds two vectors: z <- x + y
!     BLAS - like
!--------------------------------------------------------------
!     USE gckpp_Precision
      
      INTEGER :: i, M, MP1, N
      REAL(kind=dp) :: X(N),Y(N),Z(N)

      IF (N.LE.0) RETURN

      M = MOD(N,5)
      IF( M /= 0 ) THEN
         DO i = 1,M
            Z(i) = X(i) + Y(i)
         END DO
         IF( N < 5 ) RETURN
      END IF    
      MP1 = M+1
      DO i = MP1,N,5
         Z(i)     = X(i)     + Y(i)
         Z(i + 1) = X(i + 1) + Y(i + 1)
         Z(i + 2) = X(i + 2) + Y(i + 2)
         Z(i + 3) = X(i + 3) + Y(i + 3)
         Z(i + 4) = X(i + 4) + Y(i + 4)
      END DO

      END SUBROUTINE WADD
      
      
      
!--------------------------------------------------------------
      SUBROUTINE WGEFA(N,A,Ipvt,info)
!--------------------------------------------------------------
!     WGEFA FACTORS THE MATRIX A (N,N) BY
!           GAUSS ELIMINATION WITH PARTIAL PIVOTING
!     LINPACK - LIKE 
!--------------------------------------------------------------
!
      INTEGER       :: N,Ipvt(N),info
      REAL(kind=dp) :: A(N,N)
      REAL(kind=dp) :: t, dmax, da
      INTEGER       :: j,k,l
      REAL(kind=dp), PARAMETER :: ZERO = 0.0, ONE = 1.0

      info = 0

size: IF (n > 1) THEN
      
col:  DO k = 1, n-1

!        find l = pivot index
!        l = idamax(n-k+1,A(k,k),1) + k - 1
         l = k; dmax = abs(A(k,k))
         DO j = k+1,n
            da = ABS(A(j,k))
            IF (da > dmax) THEN
              l = j; dmax = da
            END IF
         END DO
         Ipvt(k) = l

!        zero pivot implies this column already triangularized
         IF (ABS(A(l,k)) < TINY(ZERO)) THEN
            info = k
            return
         ELSE   
            IF (l /= k) THEN
               t = A(l,k); A(l,k) = A(k,k); A(k,k) = t
            END IF
            t = -ONE/A(k,k)
            CALL WSCAL(n-k,t,A(k+1,k),1)
            DO j = k+1, n
               t = A(l,j)
               IF (l /= k) THEN
                  A(l,j) = A(k,j); A(k,j) = t
               END IF
               CALL WAXPY(n-k,t,A(k+1,k),1,A(k+1,j),1)
            END DO         
         END IF
         
       END DO col
       
      END IF size
      
      Ipvt(N) = N
      IF (ABS(A(N,N)) == ZERO) info = N
      
      END SUBROUTINE WGEFA


!--------------------------------------------------------------
      SUBROUTINE WGESL(Trans,N,A,Ipvt,b)
!--------------------------------------------------------------
!     WGESL solves the system
!     a * x = b  or  trans(a) * x = b
!     using the factors computed by WGEFA.
!
!     Trans      = 'N'   to solve  A*x = b ,
!                = 'T'   to solve  transpose(A)*x = b
!     LINPACK - LIKE 
!--------------------------------------------------------------

      INTEGER       :: N,Ipvt(N)
      CHARACTER     :: trans
      REAL(kind=dp) :: A(N,N),b(N)
      REAL(kind=dp) :: t
      INTEGER       :: k,kb,l

      
      SELECT CASE (Trans)

      CASE ('n','N')  !  Solve  A * x = b

!        first solve  L*y = b
         IF (n >= 2) THEN
          DO k = 1, n-1
            l = Ipvt(k)
            t = b(l)
            IF (l /= k) THEN
               b(l) = b(k)
               b(k) = t
            END IF
            CALL WAXPY(n-k,t,a(k+1,k),1,b(k+1),1)
          END DO
         END IF
!        now solve  U*x = y
         DO kb = 1, n
            k = n + 1 - kb
            b(k) = b(k)/a(k,k)
            t = -b(k)
            CALL WAXPY(k-1,t,a(1,k),1,b(1),1)
         END DO
      
      CASE ('t','T')  !  Solve transpose(A) * x = b

!        first solve  trans(U)*y = b
         DO k = 1, n
            t = WDOT(k-1,a(1,k),1,b(1),1)
            b(k) = (b(k) - t)/a(k,k)
         END DO
!        now solve trans(L)*x = y
         IF (n >= 2) THEN
         DO kb = 1, n-1
            k = n - kb
            b(k) = b(k) + WDOT(n-k,a(k+1,k),1,b(k+1),1)
            l = Ipvt(k)
            IF (l /= k) THEN
               t = b(l); b(l) = b(k); b(k) = t
            END IF
         END DO
         END IF
   
      END SELECT

      END SUBROUTINE WGESL
! End of BLAS_UTIL function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



END MODULE gckpp_LinearAlgebra

